;;;  -*- mode: LISP; Syntax: COMMON-LISP;  Base: 10 -*-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 
;;; Author      : Dan Bothell
;;; Copyright   : (c) 2010 Dan Bothell
;;; Availability: Covered by the GNU LGPL, see LGPL.txt
;;; Address     : Department of Psychology 
;;;             : Carnegie Mellon University
;;;             : Pittsburgh, PA 15213-3890
;;;             : db30@andrew.cmu.edu
;;; 
;;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun run-bold (&optional (activity t))
  (setf *save-bold* t)
  (setf *save-activity* activity)
  (play-sf (* 16 210.01) :speed 200)
  )

(defvar *conflict-sets* nil)

(defun record-conflicts (cs)
  (push cs *conflict-sets*)
  nil)

;;; record production firing data

(defvar *trial-productions* nil)
(defvar *total-firings* nil)

(defun record-firing (p)
  (push p *trial-productions*)
  
  (when (eq p 'respond-to-trial-end)
    (push (cons (chunk-slot-value cond1 value) (chunk-slot-value cond2 value)) *trial-productions*)
    (push *trial-productions* *total-firings*)
    (setf *trial-productions* nil)))


;;; record trial orders for BOLD data as well

(defvar *trial-order* nil)
(defvar *save-bold* nil)
(defvar *save-activity* nil)
(defvar *trail-number* nil)
(defvar *last-buffer-record* nil)

(defun record-trials (p)
  (when (eq p 'respond-to-trial-end)
    (push-last (cons (chunk-slot-value cond1 value) (chunk-slot-value cond2 value)) *trial-order*)
    (incf *trial-number*)
    
    (when *save-bold*
      (with-open-file (f (format nil "C:/Documents and Settings/db30/Desktop/SF/sf_model/sf_sim/data/~2,'0d.bold" *trial-number*) :direction :output :if-does-not-exist :create :if-exists :append)
        (format f "~S~%" (cons (chunk-slot-value cond1 value) (chunk-slot-value cond2 value)))
        (let ((start (* 210 (floor (mp-time) 210))))
          (dotimes (i 8)
            (format f "~S~%" (no-output (predict-bold-response (+ start 30 (* 20 i)) (+ start 50 (* 20 i)))))))))
    
    (when *save-activity*
      (with-open-file (f (format nil "C:/Documents and Settings/db30/Desktop/SF/sf_model/sf_sim/data/~2,'0d.activity" *trial-number*) :direction :output :if-does-not-exist :create :if-exists :append)
        
        (format f "~S~%" (cons (chunk-slot-value cond1 value) (chunk-slot-value cond2 value)))
        
        (let ((game-start (* 210 (floor (mp-time) 210)))
              (total-data (parse-trace-lists-for-bold (get-module bold)))
              (buffers (car (no-output (sgp :traced-buffers)))))
          
          
          (dolist (buffer buffers)
            (format f "~s~%" buffer)
            (dotimes (i 8)
              (dotimes (j 10)
                (let* ((start (+ game-start 30 (* i 20) (* j 2)))
                       (end (+ start 2)))
                  
                  (let* ((data (cdr (assoc buffer total-data)))
                         (index1 (position-if (lambda (z) (or (> (second z) start)
                                                              (and (>= (first z) start)
                                                                   (<= (first z) end)))) data))
                         (index2 (position-if (lambda (z) (> (first z) end)) data))
                         (cur-data (cond ((and index1 index2) (subseq data index1 index2))
                                         (index1 (subseq data index1))
                                         (t nil))))
                    
                    ;; increment the number of inits in the segment
                    (format f "~c~s" #\tab (cons (count-if (lambda (x) (>= (car x) start)) cur-data)
                                                 (reduce '+ cur-data :key (lambda (x) 
                                                                            (let ((s (max start (first x)))
                                                                                  (e (min end (second x))))
                                                                              (- e s)))
                                                         :initial-value 0.0))))))
              (terpri f))))))
    
    
    ;;; to make things run faster kill the old buffer trace after it has 
    ;;; been saved
    
    (setf *last-buffer-record* (btm-saved-records (get-module buffer-trace)))
    (setf (btm-current-summary (get-module buffer-trace)) nil)
    (setf (btm-saved-records (get-module buffer-trace)) nil)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun production-firing-and-key-holds (event)
  "Filter to show only production firing in the trace"
  (or (eq (evt-action event) 'production-fired)
      (eq (evt-action event) 'held-key)
      (eq (evt-action event) 'update-world)
      (eq (evt-action event) 'released-key)
      (eq (evt-action event) 'conflict-resolution)
      (eq (evt-module event) :vision)
      (and (eq (evt-action event) 'mod-buffer-chunk) (eq (car (evt-params event)) 'visual))))


(defun motor-details-only (event)
  "Filter to show only production firing in the trace"
  (when 
      (eq (evt-module event) :motor)
    (model-output "~S ~S" (evt-params event) (evt-details event))
    t))

(defun print-visual-details (event)
  (when (and (eq (evt-action event) 'mod-buffer-chunk) (eq (car (evt-params event)) 'visual))
  (aif (buffer-read 'visual)
         (model-output "time: ~8,3f Angle: ~6,2f  Speed: ~6,2f  Speed-change: ~s  Distance: ~6,2f  Vdir: ~6,2f Orient: ~6,2f hex-hit: ~s shoot-at-mine: ~s hit-fortress: ~s"
                       (evt-time event) (chunk-slot-value-fct it 'angle)
                       (chunk-slot-value-fct it 'vel)
                       (chunk-slot-value-fct it 'speed-change)
                       (chunk-slot-value-fct it 'dist)
                       (chunk-slot-value-fct it 'vdir)
                       (chunk-slot-value-fct it 'orientation)
                       (chunk-slot-value-fct it 'hex-hit)
                       (chunk-slot-value-fct it 'shoot-at-mine)
                       (chunk-slot-value-fct it 'hit-fortress-first)
                       
                       )
         (model-output "time: ~8,3f Visual buffer Empty" (evt-time event)))))


(defun show-visicon2 (event)
  (when (eq (evt-action event) 'update-world) 
    (schedule-event-relative 0 'print-visicon  :maintenance t :priority :min)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Counting shots in the imaginal buffer

(defparameter *imaginal-counting-cost* .10)

(defun count-shots ()
  (let ((current (chunk-slot-value-fct (buffer-read 'imaginal) 'shot-count)))
    
    (when (numberp current)
      (schedule-mod-buffer-chunk 'imaginal (list 'shot-count (1+ current) 'counted t) *imaginal-counting-cost* :module 'imaginal))
    (schedule-event-relative *imaginal-counting-cost* 'set-imaginal-free :module 'imaginal)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Hack for move-attention to mark the buffer as free
;;; for BOLD calculation purposes so it's not pegged at 1.

(defvar *free-tracking-hack* t)

(defvar *tracking-event* nil)


(defmethod start-tracking :after ((vis-mod vision-module))
  
  (when *free-tracking-hack* 
    (change-state vis-mod :exec 'FREE))
  
  (when *tracking-event* 
    (delete-event *tracking-event*))
  
  (setf *tracking-event*
    (schedule-event-relative (randomize-time 2) 'tracking-busy-tick :maintenance t  :module :vision :destination :vision  :details "tracking re-encode start" :output 'medium)))

(defun tracking-busy-tick (vision)
  (when (and (eq (exec-s vision) 'free) (tracked-obj-last-feat vision))
    (change-state vision :exec 'busy)
    (setf *tracking-event*
      (schedule-event-relative (randomize-time .085) 'tracking-tick-over :maintenance t  :module :vision :destination :vision  :details "tracking re-encode end" :output 'medium))))

(defun tracking-tick-over (vision)
  (when (tracked-obj-last-feat vision)
    (when (eq (exec-s vision) 'busy)
      (change-state vision :exec 'free))
    (setf *tracking-event*
      (schedule-event-relative (randomize-time 2) 'tracking-busy-tick :maintenance t  :module :vision :destination :vision  :details "tracking re-encode start" :output 'medium))))
  



(clear-all)
(define-model polygons
    
  (sgp :seed)

  (add-post-event-hook 'print-visual-details nil)
  
  (sgp :v t :trace-detail high) 
  
  (setf *conflict-sets* nil)
  
  (sgp :v nil)
  
  (sgp :save-buffer-trace t :bold-inc 2 :traced-buffers (goal visual imaginal retrieval motor-left hand-left motor-right hand-right production))
  (sgp :buffer-trace-colors ("#ff0000" "#8080ff" "#008000" "#ff8000" "#c000c0" "#e000e0" "#c0c000" "#f0f000" "#c08000"))
  
  
  ;;; don't count the individual ticks as events
  (sgp :record-ticks nil)
  
  (sgp :point-predict (goal temporal))
  
  ;;;;;; record production firings
  
  (sgp :cycle-hook record-trials)
  (setf *trial-order* nil)
  (setf *trial-number* 0)
  
  ;; standard parameters
  
  (sgp :er t :esc t :egs .2 :ol t :lf .05 :ans .2 :bll .5 :rt -.5)
  
  ;; Temporal module's noise
  
  (sgp :TIME-NOISE .005)
  
  ;; Motor timing and randomization
  
  (sgp :randomize-time 3)
  
  (sgp :MOTOR-FEATURE-PREP-TIME 0.020)
  
  (sgp :MOTOR-INITIATION-TIME .020)
  
  (sgp :fast-punch-delay .09 :default-punch-delay .125 :slow-punch-delay .16)
  
  
  (sgp :hold-until-timing .025)
  
  
  ;; visual configuration 
  
  (sgp :test-feats nil :visual-movement-tolerance 2 :do-not-harvest visual)
  
  
  
  
 ;positive: c1=2, a1=5,  b1=1
 ;negative: c2=1, a2=15, b2=1
  
  (sgp :bold-exp 5 :bold-scale 1 :neg-bold-exp 15 :neg-bold-scale 1 :bold-positive 2 :bold-negative 1 :BOLD-SETTLE 20)
  
  (sgp :use-tree t)
  
  
  (chunk-type double-tap previous)
  (chunk-type goal state turn-r dont-turn check-bonus cond1 cond2 tag mine last-count shoot-mine check-iff correct-thrust verify-vlnr)
  (chunk-type count-order current next name)
  (chunk-type game-state shot-count detect-destroy check-vlner last-bonus current-bonus counted pos)
  (chunk-type end)
  (chunk-type foe-item value game)
  
  
  (add-dm (isa count-order current 0 next 1 name "0")
          (isa count-order current 1 next 2 name "1")
          (isa count-order current 2 next 3 name "2")
          (isa count-order current 3 next 4 name "3")
          (isa count-order current 4 next 5 name "4")
          (isa count-order current 5 next 6 name "5")
          (isa count-order current 6 next 7 name "6")
          (isa count-order current 7 next 8 name "7")
          (isa count-order current 8 next 9 name "8")
          (isa count-order current 9 next 10 name "9")
          (isa count-order current 10 next 11 name "10")
          (isa count-order current 11 next 12 name "11")
          (isa count-order current 12 next 13 name "12")
          (isa count-order current 13 next 14 name "13")
          (isa count-order current 14 next 15 name "14"))
                         
  (set-all-base-levels 5)
  
  ;; position left on "asd"
  (set-hand-location left 3 4)  
  
  ;; move middle to w
  (move-a-finger (get-module :motor) 'left 'middle 1 -1.57)
  
  ;; default right for now
  (set-hand-location right 7 4)  
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;; This production starts every trial.
  ;;; 
  
  (p get-started
     "Make sure game is running and letters are visible"
     ?game>
       game-state connected
     ?goal>
       buffer empty
     =visual-search>
       isa sf-visual-location
       kind text
     ==>
     
     =visual-search> vs-dummy
     
     +goal>
       isa goal
       state attend-foes
     +visual-location>
       isa visual-location
       screen-x lowest
       :attended nil)
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;; After the trial starts attend to the letters for
  ;;; the foes.
  
  (p attend-foes
     =goal>
       isa goal
       state attend-foes
     =visual-location>
       isa visual-location
     ?visual>
       state free
     ==>
     +visual>
       isa move-attention
       screen-pos =visual-location
     +goal>
       state encode-foe)
  
  (p encode-foes
     =goal>
       isa goal
       state encode-foe
     =visual>
       isa text
       value =val
     ?imaginal>
       state free
     ==>
     +imaginal>
       isa foe-item
       value =val
       game =goal
     +visual-location>
       isa visual-location
       screen-x lowest
       :attended nil
     +goal>
       state attend-foes)
    
  (p no-more-foes
     "once the letters are read go on to rehearsal"
     =goal>
       isa goal
       state attend-foes
     ?visual-location>
     error t
     ?imaginal>
     state free
     ==>
     +goal>
       state find-conditions
       tag =goal
     
     ;; clear vision to 
     ;; - stop attending to the last letter to avoid issues 
     ;;   with automatically encoding a condition
     ;; - clear the visual-location error
     
     +visual>
       isa clear
      
     ;; make sure to clear the encoding of the last
     ;; letter to dm.
     
     -imaginal>
     
     
     ;; start rehearsing
     
     +retrieval>
       isa foe-item
       game =goal
       :recently-retrieved nil)
  
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;; Rehearse the foe letters while waiting for
  ;;; the trial conditions to appear.  Go through
  ;;; them cyclically using recently-retrieved to
  ;;; get each one once per cycle.
  
    
  (p rehearse
     =goal>
       isa goal
       state find-conditions
     =retrieval>
       isa foe-item
     ==>
     +retrieval>
       isa foe-item
       game =goal
       :recently-retrieved nil)
  
  (p rehearse-from-start
     =goal>
       isa goal
       state find-conditions
     ?retrieval>
       state error
     ==>
     +retrieval>
       isa foe-item
       game =goal
       :recently-retrieved reset)
  
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;; Attend and record the conditions for the trial
  ;;; when available.
  
  (p find-conditions
     "When conditions are visible read them"
     =goal>
       isa goal
       state find-conditions
     =visual-location>
       isa visual-location
     ?retrieval>
       state free
     ==>
     
     ;; Clear any possible lingering error from declarative
     
     +retrieval>
       isa chunk
     
     +visual-location>
       isa visual-location
       screen-x lowest
       :attended nil
     +goal>
       state attend-condition)
  
  (spp find-conditions :u 10)
  
  
  (p attend-condition
     =goal>
       isa goal
       state attend-condition
     =visual-location>
       isa visual-location
     ?visual>
       state free
     ==>
     +visual>
       isa move-attention
       screen-pos =visual-location
     +goal>
       state encode-cond
     )
  
  (p attend-cond1
     =goal>
       isa goal
       state encode-cond
       cond1 nil
       cond2 nil
     =visual>
       isa text
       value =val
     ==>
     +visual-location>
       isa visual-location
       screen-x lowest
       :attended nil
     +goal>
       cond1 =val
       state attend-condition)
  
  (p attend-cond2-+
     =goal>
       isa goal
       state encode-cond
     - cond1 nil
       cond2 nil
     =visual>
       isa text
       value =val
     ==>
     
     ;; stop attending the last condition
     
     +visual>
       isa clear
     
     ;; should probably go back to rehearsal if it's
     ;; a condition with mines, but for now just wait
     ;; to start playing the game
         
     +goal>
       cond2 =val
       state start-playing)

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;; Productions for getting a trial going once the
  ;;; ship is visible and also for restarting if the
  ;;; ship is destroyed or a trial ends.
  
  
  (p start-playing
     "once the ship is visible get going"
     =goal>
       isa goal
       state start-playing
     
     =visual-search>
       isa sf-visual-location
       kind ship
     
     ?visual>
     state free
     ?imaginal>
     state free
     ==>
     =visual-search> vs-dummy
     
     +goal>
       state start
     +visual>
       isa attend-and-track
       screen-pos =visual-search
     
     ;; initialize the shot count
     ;; should probably restore it or re-read it if this happens
     ;; due to ship destruction...
     
     +imaginal>
       isa game-state
       shot-count 0
     pos 0
     
     ;; start the temporal module ticking
     
     +temporal>
       isa time
     )
  
  
  (p detect-destroyed-1
     "If the ship is destroyed wait for the ship to respawn and restart"
     =goal>
       isa goal
     - state start-playing
     
     ;; If there is an explosion visible regardless of
     ;; current attention
     
     =visual-search>
       isa sf-visual-location
       kind explosion
     
     ?manual>
       processor free
     
     ?visual>
       state free
     ==>
     =visual-search> vs-dummy
     
     ;; reset all of the goal flags and wait for the ship to 
     ;; be visible
     
     +goal>
       state start-playing
       mine nil
       turn-r nil
       dont-turn nil
     
     ;; make sure the visual state is sane and stop pressing
     ;; any keys
     
     -visual-location>
     
     +visual>
       isa move-attention
       screen-pos =visual-search
     
     +manual>
      isa release-all-fingers
     )
  
  
  
  (p detect-destroyed-2
     "Like the previous one but catches things when the ship's explosion is attended"
     
     =goal>
       isa goal
     - state start-playing
     
     ;; Tracking the ship will lead to the explosion being encoded
     ;; directly in visual
     
     =visual>
       isa sf-object
       value explosion
     
     ?manual>
       processor free
     ?visual>
       state free
     ==>
     +goal>
       state start-playing
       turn-r nil
       dont-turn nil
       mine nil
       
     -visual-location>
     -visual>
     
     +manual>
      isa release-all-fingers
     )

  (p detect-trial-end
     "If a trial is over get ready to clear things out"
     =goal>
       isa goal
     =visual-search>
       isa sf-visual-location
       kind new-trial-end
     ?manual>
       processor free
     ?visual>
       state free
     ==>
     =visual-search> vs-dummy
     +visual>
       isa move-attention
       screen-pos =visual-search
     +goal>
       isa end
     
     -visual-location>
     
     +manual>
      isa release-all-fingers
     )
  
  ;; set U rediculously high to avoid any issues with
  ;; missing it...
  
  (spp detect-trial-end :u 10000)
  
  
  (p respond-to-trial-end
     "once all of the modules are settled clear things out prior to the next trial"
     
     =goal>
       isa end
     =visual>
       isa trial-end
     ?manual>
       processor free
     ?imaginal>
       state free
     ==>
     
     ;; clear out current info
     
     -goal>
     -imaginal>
     )
  
  
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;; Always start by getting the ship up to speed 
  ;;; when it starts from the beginning.
  
  
  (p hold-thrust
     =goal>
       isa goal
       state start
     =visual>
       isa ship
     ?manual>
       processor free
     ?finger-check>
       left-middle free
   ==>
     +goal>
       state up-to-speed
     +manual>
       isa hold-key
       hand left
       finger middle)
  
  
  (p release-thrust
     "release the thrust and start the main 'play' routine"
     
     =goal>
       isa goal
       state up-to-speed
     =visual>
       isa ship
       >= vel .9
     ?manual>
       processor free
     ?finger-check>
       left-middle busy
     ==>
     
     +goal>
       state play
     
     +manual>
       isa release-key
       hand left
       finger middle)
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;; The main orbiting productions do most of the work.
  ;;; They're broken into cases based on the current 
  ;;; situation and some cases are more important than 
  ;;; others.
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;; Whenever the ship is turned too far to the left of
  ;;; the fortress turn to the right.  This is important
  ;;; because that's how the ship starts and it needs to
  ;;; correct that.
  ;;;
  ;;; It doesn't want to do that however if it's currently
  ;;; trying to shoot a mine, shoot the fortress or already
  ;;; correcting such a turn.
  
  (p hold-right
     =goal>
       isa goal
       state play
       turn-r nil
       dont-turn nil
       mine nil
     =visual>
       isa ship
       > angle 30
     ?manual>
       processor free
     ?finger-check>
       left-index free
    ==>
     +goal>
       turn-r t
     +manual>
	 isa hold-key
       hand left
       finger index)

  (p release-right
     =goal>
       isa goal
       state play
       turn-r t
     =visual>
       isa ship
       <= angle 6
     ?manual>
       processor free
    ==>
     +goal>
       turn-r nil
     +manual>
       isa release-key
       hand left
       finger index)
  
  
  ;;;
  ;;; Some productions to just note the ship position as it moves
  ;;; to provide some imaginal activity in the orbit condition
  
  (p note-orientation-change
     =goal>
     isa goal
     state play
     =imaginal>
     isa game-state
     pos =pos
     =visual>
     isa ship
     orientation =now
     !eval! (> (abs (- =now =pos)) 90)
     ?imaginal>
     state free
     ==>
     +imaginal>
     pos =now)
     
  
  
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  
  ;;; Productions in this section are going to be named
  ;;; based on the action, the current ship speed, and the length of
  ;;; the key press for ease of reading the trace.
  ;;; They'll be of the form:
  ;;;
  ;;; [thrust | left | right]-[norm | fast | slow]-[short | default | long]
  
  
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;; When the model is flying at a "normal" speed these
  ;;; productions keep it orbiting.  Normal is defined as
  ;;; between 1.0 and 1.7.  1.0 is the velocity score's
  ;;; lower limit, but 1.7 is below the upper scoring
  ;;; threshold.
  ;;;
  
  
  (p thrust-norm-default
     =goal>
       isa goal
       state play
     =visual>
       isa ship
     
     ;; speed in the normal range
     <= vel 1.7
     >= vel 1.0
     
     ;; not too close, "past" the fortress
     ;; and pointing toward it
     
     > dist 93
     >= vdir 92
     <= angle 2
     >= angle -4
      
     ?manual>
       processor free
     
     ;; dont thrust this long if turning
     
     ?finger-check>
       left-middle free
       left-index  free
       left-ring   free
     ==>
     +manual>
       isa delayed-punch
       hand left
       finger middle
     )
  
  
  (p right-norm-short
     =goal>
       isa goal
       state play
       dont-turn nil
     =visual>
       isa ship
     
     <= vel 1.7
     >= vel 1.0
     
     ;; angle slighty away from
     ;; fortress and past it
     
     >= angle 5
     <= angle 10
     > vdir 92
      
     ?manual>
       processor free
     ?finger-check>
       left-index free
    ==>
     +manual>
       isa delayed-punch
       hand left
       finger index
       delay fast)
  
  
  
  
   (p right-norm-default
     =goal>
       isa goal
       state play
       dont-turn nil
     =visual>
       isa ship
     <= vel 1.7
     >= vel 1.0
 
      ;; angled farther away and/or
      ;; not quite as far past
      
     >= angle 5
     <= angle 20
      
      > vdir 88
      
     ?manual>
       processor free
     ?finger-check>
       left-index free
    ==>
     +manual>
       isa delayed-punch
       hand left
       finger index)
  
  
  (p right-norm-long
     =goal>
       isa goal
       state play
       dont-turn nil
     =visual>
       isa ship
       <= vel 1.7
       >= vel 1.0
     
     ;; angle way off and don't
     ;; care about the relative position
    
       >= angle 15
       <= angle 30
 
     ?manual>
       processor free
     ?finger-check>
       left-index free
    ==>
     +manual>
       isa delayed-punch
       hand left
       finger index
       delay slow
     )
    
  (p left-norm-default
     =goal>
       isa goal
       state play
       dont-turn nil
     =visual>
       isa ship
     <= vel 1.7
     >= vel 1.0
     
     ;; lefts just correct
     ;; when too far right
     ;; ignoring positional
     ;; info and no need for
     ;; a range of responses
     
     < angle -20
     
     ?manual>
       processor free
     
     ;; Like thrusts don't do this
     ;; if any other movement actions are 
     ;; ongoing
     
     ?finger-check>
       left-ring free
       left-index free
       left-middle free
     ==>
     
     +goal>
     dont-turn nil
     +manual>
       isa delayed-punch
       hand left
       finger ring
       ;delay slow
     )
  
  ;;; Prefer the extreme corrective lengths
  ;;; to the overlapping middle region.
  
  (spp right-norm-short :u 2)
  (spp right-norm-default :u 1)
  (spp right-norm-long :u 2)
  
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;; When the model is flying fast these productions
  ;;; handle the orbiting to try to slow it down
  ;;; to the normal range.  Too fast being a velocity
  ;;; above 1.7.
  
  
  (p thrust-fast-long
     "slow down with the turn"
     =goal>
       isa goal
       state play
         
     =visual>
       isa ship
      > vel 1.7
     
     ;; if it's moving fast let it
     ;; drift a little further out
     ;; and point closer in before
     ;; thrusting to correct
     
      > dist 99
      >= vdir 93
      <= angle 0
      >= angle -4
     
     ?manual>
       processor free
     
     ?finger-check>
      ; left-ring free
      ; left-index free
       left-middle free
   
     ==>
          +goal>
     dont-turn nil
     +manual>
       isa delayed-punch
       hand left
       finger middle
       delay slow)
  

  (p right-fast-default
     =goal>
       isa goal
       state play
       dont-turn nil
     =visual>
       isa ship
       > vel 1.7
     
     ;; medium offset angle
     
       >= angle 4
       <= angle 15
    
     ?manual>
       processor free
     ?finger-check>
       left-index free
    ==>
     +manual>
       isa delayed-punch
       hand left
       finger index)
  
  
  (p left-fast-short
     =goal>
       isa goal
       state play
       dont-turn nil
     =visual>
       isa ship
       
     > vel 1.7
     
     < angle -20
      
     ?manual>
       processor free
     
     ;; don't correct left if still
     ;; turning from some other action
     
     ?finger-check>
        left-ring free
        left-index free
      ;  left-middle free

     ==>
     +goal>
     dont-turn nil
     +manual>
       isa delayed-punch
       hand left
       finger ring
       delay fast)
  
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;; When the model is going too slow turn into
  ;;; the orbit before thrusting to gain some speed.
  ;;; Too slow is below 1.0 which would be losing
  ;;; points for velocity.
  
    
  (p thrust-slow-long
     =goal>
       isa goal
       state play
     
     =visual>
       isa ship
     
     ;; if extra slow give more thrust
     
     < vel .7
      
     ;; check for a good distance,
     ;; that it's past the fortress,
     ;; and that it's pointing slightly
     ;; to the left.
     
      > dist 85
      >= vdir 93
      <= angle 5
      > angle 0
     
     ?manual>
       processor free
     
     ?finger-check>
       left-middle free
       left-index free
     ==>
          +goal>
     dont-turn nil
     +manual>
       isa delayed-punch
       hand left
       finger middle
       delay slow)
  
  
  (p thrust-slow-default
     =goal>
       isa goal
       state play
     
     =visual>
       isa ship
       
       ;; not too slow needs just a little boost
       
       < vel 1.0
       >= vel .7
       
      > dist 85
      >= vdir 93
      <= angle 5
      > angle 0
     
     ?manual>
       processor free
       
     ?finger-check>
       left-middle free
       left-index free
     ==>
          +goal>
     dont-turn nil
     +manual>
       isa delayed-punch
       hand left
       finger middle
       delay slow)
  
  (p right-slow-default
     =goal>
       isa goal
       state play
       dont-turn nil
     =visual>
       isa ship
     < vel 1.0
       
     >= angle 7
   
     ?manual>
       processor free
     
     ;; dont correct if already turning
     
     ?finger-check>
       left-index free
       left-ring free
    ==>
     +manual>
       isa delayed-punch
       hand left
       finger index)
  

  (p left-slow-short
     =goal>
       isa goal
       state play
       dont-turn nil
     =visual>
       isa ship
     
     < vel 1.0
     
     ;; If past fortress turn
     ;; out when going slow
     
     >= vdir 93
     < angle 0
     >= angle -5
     
     ?manual>
       processor free
     
     ?finger-check>
       left-ring free
       left-index free
       left-middle free
   
     ==>
     +goal>
     dont-turn nil
     +manual>
       isa delayed-punch
       hand left
       finger ring
       delay fast)
  
  (p left-slow-default
     =goal>
       isa goal
       state play
       dont-turn nil
     =visual>
       isa ship
       < vel 1.0
     
     ;; make a bigger correction 
     ;; out when needed
     
       >= vdir 93
       < angle -5
     
     ?manual>
       processor free
     
     ?finger-check>
       left-ring free
       left-index free
       left-middle free
     ==>
     +goal>
     dont-turn nil
     +manual>
       isa delayed-punch
       hand left
       finger ring)

  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;; One type of error the model can make is to bounce off of the
  ;;; inner hex.  These productions detect that and attempt to correct
  ;;; with a simple turn left followed by a thrust to get back up to 
  ;;; speed in the orbit.  Correcting the bounce suppresses most
  ;;; other actions since it changes the goal state.
  ;;; Hitting the hex is a property directly available in the ship's
  ;;; visual chunk instead of having to determine that somehow.
  
  (p detect-bounce
     =goal>
       isa goal
       state play
     =visual>
       isa ship 
       hex-hit t
     ?manual>
       processor free
     ?finger-check>
       left-middle free
     ==>
     ;; suppress any other turns 
     ;; and clear a right turn if its
     ;; ongoing
     
     +goal>
       state correcting-bounce
       dont-turn t
       turn-r nil
     
     ;; stop any current actions
     +manual>
       isa release-all-fingers)
  
  
  ;; detecting the bounce is a pretty high
  ;; priority action.
  ;; Doesn't seem to conflict however because
  ;; all the angles and stuff is way off for
  ;; the other navigation productions.
  
  (spp detect-bounce :u 15)
  
  
  (p correct-bounce-turn
     =goal>
       isa goal
       state correcting-bounce
     ?finger-check>
       left-middle free
       left-ring free
       left-index free
     ?manual>
       processor free
     ==>
     +goal>
       state bounce-hold
     +manual>
       isa hold-key
       hand left
       finger ring)
  
  (p correct-bounce-release
     =goal>
       isa goal
       state bounce-hold
          
     =visual>
       isa ship
       > angle 35
     
     ?manual>
       processor free
     ==>
     +goal>
       state bounce-thrust
     +manual>
       isa release-key
       hand left
       finger ring)
  
  (p correct-bounce-thrust
     =goal>
       isa goal
       state bounce-thrust
          
     =visual>
       isa ship
       > angle 40
     
     ?manual>
       processor free
     ==>
     +goal>
       state bounce-thrust-held
     +manual>
       isa hold-key
       hand left
       finger middle)

  (p correct-bounce-done
     =goal>
       isa goal
       state bounce-thrust-held
          
     =visual>
       isa ship
       > vdir 0
       > vel 1.0
     
     ?manual>
       processor free
     ==>
     +goal>
       state play
       dont-turn nil
     +manual>
       isa release-key
       hand left
       finger middle)

  
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;; These productions are for shooting at the fortress while in a
  ;;; typical orbiting situation.
  ;;; It counts the number of hits to determine when the fortress is
  ;;; vulnerable instead of checking the vlnr score.
  ;;; It also uses the temporal module to space out the shooting.
  ;;; It waits 13 ticks between the "slow shots" which should
  ;;; be enough to space keep it above the 250ms reset time 
  ;;; (~270ms for the temporal delay plus the motor action time).
  ;;;
  ;;; The determination of whether or not it's pointing at the
  ;;; fortress is just a crude angle check with a step at different
  ;;; distances.  It could be made more precise, but this seems 
  ;;; to work sufficiently.
  ;;;
  ;;; One problem with this is that if it ever does get a reset
  ;;; on the fortress due to shooting too quickly then it will
  ;;; be stuck because it won't reset its count.  Need to add 
  ;;; some sort of sanity check eventually -- perhaps a further
  ;;; counting such that if it gets too high it reads vlnr or
  ;;; count the "double taps" and if there's a double tap that
  ;;; hits and doesn't kill check the vlnr.

  (p shoot-slow-close
     =goal>
       isa goal 
     =visual>
       isa ship 
     
       < angle 14
       > angle -14
       <= dist 100
     ?manual>
       processor free
     
     =imaginal>
       isa game-state
       < shot-count 10
       shot-count =count
     
     ?imaginal>
     state free
     
     =temporal>
       isa time
       > ticks 13
     
     =visual-search>
       isa sf-visual-location
       kind fortress
    
     ?finger-check>
       right-index free   
       left-index free
       left-ring free
     ==>
     =visual-search> vs-dummy
     =imaginal>
     counted nil

     +temporal>
       isa time
     +manual>
       isa delayed-punch
       hand right
       finger index
     )
  
  (p shoot-slow-far
     =goal>
     isa goal
     =visual>
       isa ship 
     
     ;; a little tighter angle at a greater distance
     
     < angle 11
     > angle -11
     > dist 100
     
     ?manual>
     processor free
     ?imaginal>
     state free
     
     =imaginal>
       isa game-state
       < shot-count 10
     shot-count =count
     =temporal>
       isa time
       > ticks 13
     =visual-search>
       isa sf-visual-location
       kind fortress
    
     ?finger-check>
       right-index free   
       left-index free
       left-ring free
     ==>
     =visual-search> vs-dummy
     =imaginal>
     counted nil
     +temporal>
       isa time
     +manual>
       isa delayed-punch
       hand right
       finger index)
  
  
  (p shoot-fast-first-close
     =goal>
       isa goal 
     =visual>
       isa ship 
     
     ;; because there are two shots without turning
     ;; the angle can't be too far to the left
     ;; or the second one will miss
     
       < angle 4
       > angle -12
       <= dist 100
     
     ?manual>
       processor free
     ?finger-check>
       right-index free   
       left-index free
       left-ring free
     =imaginal>
       isa game-state
       >= shot-count 10
       check-vlner nil
     =visual-search>
       isa sf-visual-location
       kind fortress

     ==>
     =visual-search> vs-dummy
     
     ;; make sure the model doesn't
     ;; turn away and potentially miss
     ;; the next shot
     
     +goal>
       dont-turn t
     
     =imaginal>
       check-vlner double
       detect-destroy t
    
     +manual>
       isa delayed-punch
       hand right
       finger index
       delay .07)
  
    
  (p shoot-fast-first-far
     =goal>
       isa goal 
     =visual>
       isa ship 
       < angle 2
       > angle -9
       > dist 100
     
     ?manual>
       processor free
     ?finger-check>
       right-index free   
       left-index free
       left-ring free
     

     
     =imaginal>
       isa game-state
       >= shot-count 10
       check-vlner nil
     =visual-search>
       isa sf-visual-location
       kind fortress
        
     ==>
     =visual-search> vs-dummy
     
     +goal>
       dont-turn t
     
     =imaginal>
       check-vlner double
       detect-destroy t

     +manual>
       isa delayed-punch
       hand right
       finger index
       delay .07)
  
     
  (p shoot-fast-second
     =goal>
       isa goal 
     =visual>
       isa ship 
     
     ?manual>
       processor free
     
     ?finger-check>
       right-index free   
       left-index free
       left-ring free
     
     =imaginal>
       isa game-state
       check-vlner double
     =visual-search>
       isa sf-visual-location
       kind fortress

     ==>
     =visual-search> vs-dummy
     
     +goal>
       dont-turn nil
     
     =imaginal>
       check-vlner nil
     +manual>
       isa delayed-punch
       hand right
       finger index
       delay fast)
  
  ;; early shooting is of a lower priority than
  ;; the orbiting actions, but the second of
  ;; the quick shots for a kill is important
  
  (spp shoot-slow-close :u -4)
  (spp shoot-slow-far :u -4)
  
  (spp shoot-fast-first-close :u -1)
  (spp shoot-fast-first-far :u -1)
  
  (spp shoot-fast-second :u 25)

  ;; to count the hits the model needs to
  ;; see them and they're an explicit feature
  ;; in the visicon
  
  (p detect-hit
     =visual-search>
       isa sf-visual-location
       kind fortress-hit
     ?retrieval>
       state free
     ?imaginal>
       state free
     =imaginal>
       isa game-state
       shot-count =count
     < shot-count 10
       counted nil
     ==>
     =imaginal>
     =visual-search> vs-dummy
     +imaginal-action>
     isa generic-action
     action count-shots
     
     )
  
  ;; Detecting and counting fortress hits are fairly high priority
  ;; actions.
  
  (spp detect-hit :u 25)
     
  ;; If a fortress is destroyed that is again a
  ;; separate feature (the explosion is actually a
  ;; real feature in the game as well).
  
  (p detect-destroyed-fortress
     =goal>
       isa goal
     =visual-search>
       isa sf-visual-location
       kind fortress-explosion
     =imaginal>
       isa game-state
     detect-destroy t
     ?imaginal>
     state free
     ==>
     =visual-search> vs-dummy
     +goal>
     dont-turn nil
     +imaginal>
       shot-count 0
       detect-destroy nil
       check-vlner nil
     )
  
  ;; A fortress explosion persists for a while (1s)
  ;; so it's not as critical to get it right away.
  
  (spp detect-destroyed-fortress :u 5)
  
  
 
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;; In the fortress conditions the model needs to check the
  ;;; bonus symbol when it appears.  The model will only check
  ;;; the bonus symbol when it is in a 'stable' situation.  For
  ;;; now stable is when it thrusts at the start of a side in
  ;;; the orbiting since that's the point most likely to be the
  ;;; farthest from the next required adjustment.
  ;;; It will attend to the symbol, compare it to the previous one,
  ;;; and then save it in a slot of the chunk in the imaginal buffer.
  ;;; If it was a matching bonus symbol then it will hit the
  ;;; key to get the bonus.
  
  
  (p detect-bonus
     "Note that a new bonus symbol is available to attend"
     =goal>
       isa goal 
       check-bonus nil
     
     =visual-search>
       isa sf-visual-location
       kind new-bonus
     
     =imaginal>
       isa game-state
     
     ==>
     =visual-search> vs-dummy
     =imaginal>
     +goal>
       check-bonus check)
  
  
  (p check-bonus
     "When thrusting its safe to go check"
     =goal>
       isa goal
       state play
       check-bonus check
    
     ?finger-check>
       left-middle busy
     
     ==>
     +goal>
       check-bonus attend
     +visual>
       isa move-attention
       screen-pos bonus-loc)
  
  
  (p attend-bonus-matching-target
     =goal>
       isa goal
       check-bonus attend
     =visual>
       isa bonus
       value "$"
     =visual-search>
       isa visual-location
       kind ship
     ?visual>
       state free
     =imaginal>
       isa game-state
       last-bonus "$"
     ?manual>
       processor free
     ?finger-check>
       right-ring free
     ==>
     =visual-search> vs-dummy
     =imaginal>
     +manual>
       isa delayed-punch
       hand right
       finger ring
     
     +visual>
       isa attend-and-track
       screen-pos =visual-search
     +goal>
       check-bonus nil)
  
  (p attend-bonus-mismatch-target
     =goal>
       isa goal
       check-bonus attend
     =visual>
       isa bonus
       value "$"
     =visual-search>
       isa visual-location
       kind ship
     ?visual>
       state free
     =imaginal>
       isa game-state
     - last-bonus "$"
     ==>
     =visual-search> vs-dummy
     =imaginal>
       last-bonus "$"
     +visual>
       isa attend-and-track
       screen-pos =visual-search
     +goal>
       check-bonus nil)
  
   (p attend-bonus-non-target
     =goal>
       isa goal
       check-bonus attend
     =visual>
       isa bonus
        value =val
      - value "$"
     =visual-search>
       isa visual-location
       kind ship
     ?visual>
       state free
     =imaginal>
       isa game-state
      ==>
     =visual-search> vs-dummy
     =imaginal>
       last-bonus =val     
     +visual>
       isa attend-and-track
       screen-pos =visual-search
     +goal>
       check-bonus nil)
  
  (p missed-bonus
     "Nothing to attend"
     =goal>
       isa goal
       check-bonus attend
     =visual-search>
       isa visual-location
       kind ship
     ?visual>
       state free
       error t
     ==>
     =visual-search> vs-dummy
     +visual>
       isa attend-and-track
       screen-pos =visual-search
     +goal>
       check-bonus nil)
  
    (p missed-bonus-on-ship
       "Attended ship because it was near bonus spot"
       =goal>
         isa goal
         check-bonus attend
     =visual>
       isa ship
       
     =visual-search>
       isa visual-location
       kind ship
     ==>
     =visual-search> vs-dummy
       
     +visual>
       isa attend-and-track
       screen-pos =visual-search
     +goal>
       check-bonus nil)
  
  ;; Detecting the new bonus symbol is important
  
  (spp detect-bonus :u 10)
  
  ;; attending it is not particularly high priority
  
  (spp check-bonus :u 1)
  
 
  ;; This case is very high priority because
  ;; it needs to start tracking the ship again instead
  ;; of just continuing to "see" the ship in that spot
  ;; and not acting on it (which the orbiting productions
  ;; might do otherwise).
  
  (spp missed-bonus-on-ship :u 100)
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;; This section deals with the mine.
  ;;; 
  ;;; When the model detects the mine it checks the iff
  ;;; info and then attempts to retrieve whether or not
  ;;; the mine is a foe.  If so, then it will hit the
  ;;; iff key twice to activate it.  It also does a mental
  ;;; projection of the mine and ship positions to determine
  ;;; if and when it can shoot it.  That's done through a
  ;;; request to the imaginal module which handles all of
  ;;; the math involved (except for accounting for the time
  ;;; that it takes the shot to get from the ship to the 
  ;;; mine which may be important).
  ;;;
  ;;; Based on that projection the model will then either
  ;;; ignore the mine (there's no chance to hit it) or 
  ;;; take the appropriate turn to shoot it if necessary.
  ;;; Part of the prediction is to determine how close
  ;;; the mine can get before an action is required so that
  ;;; when there is a fortress the model can continue to
  ;;; shoot at it until it needs to deal with the mine.
  ;;;
  ;;; Another component of the prediction is a distance
  ;;; at which the mine should be destroyed.  If it gets
  ;;; closer than that it will revaluate the prediction
  ;;; and try again to make the necessary corrections.
  ;;;
  ;;; If the mine gets too close the model will just shoot
  ;;; at it regardless of whether or not it has completed
  ;;; a necessary iff.
  ;;;
  ;;; After handling the mine the model will return to
  ;;; orbiting the fortress.  If the distance and current
  ;;; vdir are ok, then it will just return to the standard
  ;;; orbiting.  However, if either distance or vdir are
  ;;; "large" then it will attempt to get closer before 
  ;;; going back to the normal orbitting.  Those corrections
  ;;; still need a little adjusting to "look" good...
  
  (p see-mine-onset
     =goal>
       isa goal
       state play
     =visual-search>
       isa visual-location
       kind mine
     =visual>
       isa ship
     
     ;; only do the prediction once not turning
     ;; to get a stable result
     
     ?finger-check>
       left-ring free
       left-index free
     
     ;; save the current count since the prediction
     ;; will overwrite it
     
     =imaginal>
       isa game-state
       shot-count =count
     
     ?imaginal>
       state free
     
     ==>
     
     =visual-search> vs-dummy
     
     ;; clear out the current imaginal info
     
     -imaginal>
     
     ;; request the prediction
     
     +imaginal-action>
       isa generic-action
       action predict-mine-shooting
     
     ;; change the state and set the flags
     ;; with the mine info and the last count
     ;; of fortress vulnerability
     
     +goal>
       state attend-mine
       last-count =count
       mine t
       shoot-mine nil
     
     ;; check the iff info
     
     +visual>
       isa move-attention
       screen-pos iff-loc)
  
  
   (p encode-iff-letter
     =goal>
       isa goal
       state attend-mine
       tag =game
     =visual>
       isa text
       value =letter
     =visual-search>
       isa visual-location
       kind ship
     ==>
     =visual-search> vs-dummy
     +goal>
       state recall-iff
       check-iff t
     +retrieval>
       isa foe-item
       value =letter
       game =game
     +visual>
       isa attend-and-track
       screen-pos =visual-search)
  
    (p engage-iff
     =goal>
       isa goal
       state recall-iff
     =retrieval>
       isa foe-item
     ?manual>
       processor free
     ?finger-check>
       right-middle free
     ==>
       +temporal>
       isa time
       +manual>
       isa delayed-punch
       hand right
       finger middle
       delay slow
       
       ;; set a new goal to ensure
       ;; the second iff key is the only
       ;; next action
       
     +goal>
       isa double-tap
       previous =goal
     
     =goal>
       state play
       shoot-mine t
       check-iff nil)
  
  (p double-tap-iff
     =goal>
       isa double-tap
       previous =g
     ?manual>
       processor free
     ?finger-check>
     right-middle free
     =temporal>
     isa time
     > ticks 13
     ==>
     +manual>
       isa delayed-punch
       hand right
       finger middle
       ;delay slow
     
     ;; restore the previous goal
     =goal> =g)
  
  (p no-iff-needed
     =goal>
       isa goal
       check-iff t
     ?retrieval>
       state error
     ==>
     +goal>
       check-iff nil
       shoot-mine t)
  
  
  
  ;; if there's a fortress don't adjust for the
  ;; mine until it's close
  
  (p adjust-mine-heading-with-fortress
     =goal>
       isa goal
       cond2 "+fort"
     =visual>
       isa ship
       ;; focus on fortress until mine is close
       <= mine-dist =dist
     =visual-search>
       isa visual-location
       kind mine
     =imaginal>
       isa mine-heading
       action turn
       finger =finger
       delay =delay
       dist =dist
     ?manual>
       processor free
     ?finger-check>
       left-index free
       left-ring free
    
     ==>
     =goal>
     dont-turn t
     
     =visual-search> vs-dummy
     
     ;; clear the turn indication
     =imaginal>
       action drift
     
     +manual>
       isa delayed-punch
       hand left
       finger =finger
       delay =delay)
  
  
  ;; without a fortress present adjust for the mine
  ;; as soon as possible
  
  (p adjust-mine-heading-no-fortress
     =goal>
       isa goal
       cond2 "-fort"
     =visual>
       isa ship
     =visual-search>
       isa visual-location
       kind mine
     =imaginal>
       isa mine-heading
       action turn
       finger =finger
       delay =delay
       dist =dist
     ?manual>
       processor free
     ?finger-check>
       left-index free
       left-ring free
    
     ==>
     =goal>
     dont-turn t
     =visual-search> vs-dummy
     
     =imaginal>
       action drift
     +manual>
       isa delayed-punch
       hand left
       finger =finger
       delay =delay)
  
  
  ;; if the mine is too close and a correction has
  ;; been made if required repredict the shooting
  
  (p reassess-mine
     =goal>
       isa goal
     =visual-search>
       isa visual-location
       kind mine
     =imaginal>
       isa mine-heading
       action drift
       hit-dist =delay
              
     =visual>
       isa ship 
       < mine-dist =delay
       
     ?finger-check>
       left-ring free
       left-index free
       
     ?imaginal>
       state free
       
     ==>
     =visual-search> vs-dummy
     
     =imaginal>
     
     +goal>
       state handle-mine
     
     +imaginal-action>
       Isa generic-action
       action predict-mine-shooting)
  
  ;; If the model determines it
  ;; can't hit the mine in the fortress
  ;; condition it'll just go back to shooting
  ;; at the fortress
  
  (p cant-hit-mine-ignore 
     =goal>
       isa goal
       cond2 "+fort"
       mine t
     =imaginal>
       isa mine-heading
       action panic
     ?imaginal>
       state free
     ==>
         
     +imaginal>
       isa game-state
       shot-count unknown 
     pos 0
     
     +goal>
       state play
       mine nil
       dont-turn nil)
  
  ;;; Handle shooting at the fortress if available
  ;;; and the mine when possible.  It won't kill
  ;;; a fortress however if the mine is on screen.
  
  ;; fortress shooting productions which
  ;; check the last recorded count in the
  ;; goal since the imaginal now has the
  ;; mine prediction
  
  (p shoot-slow-close-with-mine
     =goal>
     isa goal 
     mine t
     < last-count 9
     =visual>
       isa ship 
       < angle 14
       > angle -14
       <= dist 100
     ?manual>
       processor free
     =temporal>
       isa time
       > ticks 13
      =visual-search>
       isa sf-visual-location
       kind fortress
    
     ?finger-check>
       right-index free   
       left-index free
       left-ring free
     ==>
     =visual-search> vs-dummy
     +temporal>
       isa time
     +manual>
       isa delayed-punch
       hand right
       finger index)
  
  (p shoot-slow-far-with-mine
     =goal>
       isa goal
       mine t
     < last-count 9
     
     =visual>
       isa ship 
       < angle 11
       > angle -11
       > dist 100
     ?manual>
       processor free
      =temporal>
       isa time
       > ticks 13
     =visual-search>
       isa sf-visual-location
       kind fortress
    
     ?finger-check>
       right-index free   
       left-index free
       left-ring free
     ==>
     =visual-search> vs-dummy
     +temporal>
       isa time
     +manual>
       isa delayed-punch
       hand right
       finger index)
  
  
  ;; mine shooting productions.
  ;; basically shoot the mine as soon as possible
  ;; after iff (if needed) or just shoot at it when
  ;; it gets critically close.
  ;; The ship being pointed at the mine is a property
  ;; of the ship visual chunk and the model will only
  ;; shoot if it's pointed in the right direction.
  
  (p shoot-close-mine-ignoring-iff
     =goal>
       isa goal
     =visual-search>
       isa visual-location
       kind mine
     =visual>
       isa ship
       shoot-at-mine t
     < mine-dist 50
     ?manual>
       processor free
     ?finger-check>
       right-index free
     ==>
     =visual-search> vs-dummy
     +manual>
       isa delayed-punch
       hand right
       finger index
       delay fast)
  
  (p shoot-mine
     =goal>
       isa goal
       shoot-mine t
     =visual-search>
       isa visual-location
       kind mine
     =visual>
       isa ship
       shoot-at-mine t
       hit-fortress-first nil
     ?manual>
       processor free
     ?finger-check>
       right-index free
     ==>
     =visual-search> vs-dummy
     +manual>
     isa delayed-punch
     hand right
     finger index
     ;delay fast
     )
    
  ;; Mine handling productions are all high priority
  
  (spp see-mine-onset :u 50)
  
  ; don't compete
  ;(spp engage-iff :u 50)
  ;(spp double-tap-iff :u 50)
  
  (spp adjust-mine-heading-with-fortress :u 50)
  (spp adjust-mine-heading-no-fortress :u 50)
  
  (spp reassess-mine :u 25)
  
  (spp cant-hit-mine-ignore :u 50)

  (spp shoot-close-mine-ignoring-iff :u 50)
  (spp shoot-mine :u 50)

  
  ;;; Detect the mine's disapearance and go back to 
  ;;; orbiting the inner hex.  The mine's disapearance
  ;;; is an explicit visual feature (dead-mine) to
  ;;; make detection easier.
  
  
  (p mine-gone-check-path
     =goal>
       isa goal
       mine t
     =visual-search>
       isa visual-location
       kind dead-mine
     ==>
     =visual-search> vs-dummy
          
     +goal>
       state correct
       mine nil)
  
  
  (p mine-gone-orbit-ok
     =goal>
       isa goal
       state correct
     =visual>
       isa ship
     <= dist 125
     >= angle 18
     <= angle -18
     <= vdir 125
     ?imaginal>
       state free

          ?finger-check>
       left-index free
       left-middle free
     left-ring free
     ==>
     
     ;; create a new game-state and prepare to
     ;; read the current vlnr
     
     +imaginal>
       isa game-state
       shot-count unknown 
       pos 0
     ;; return the goal to the standard orbit
     +goal>
       state play
       dont-turn nil)
    
  
  
    (p mine-gone-turn-right
     =goal>
       isa goal
       state correct
     =visual>
       isa ship
     < dist 125
     > angle 18
       
       ?imaginal>
       state free
       
     ?finger-check>
       left-index free
       left-middle free
       left-ring free
     ==>
       
            +imaginal>
       isa game-state
       shot-count unknown 
       pos 0
     +manual>
     isa hold-until
     hand left
     finger index
     slots (angle)
     values (9)
     
      +goal>
       state play
       dont-turn nil

     )
     
 (p mine-gone-turn-left
     =goal>
       isa goal
       state correct
     =visual>
       isa ship
     < dist 125
     < angle -18
     
     ?imaginal>
       state free
    
         ?finger-check>
       left-index free
       left-middle free
    left-ring free
    ==>
         +imaginal>
       isa game-state
       shot-count unknown 
       pos 0
    +manual>
     isa hold-until
     hand left
     finger ring
     slots (angle)
     values (-6)
     
      +goal>
       state  play
     dont-turn nil
)
  
  
  ;; if the model has gone too far out try to
  ;; correct that too
  
  
  (p mine-gone-bad-dist-angle-ok
     =goal>
       isa goal
       state correct
     =visual>
       isa ship
     > dist 125
     >= angle -20
     <= angle 20
     ?imaginal>
       state free
     
         ?finger-check>
       left-index free
       left-middle free
     left-ring free
     ==>
     =visual-search> vs-dummy
     
     +imaginal-action>
     isa generic-action
     action reorbit-from-dist
     
      +goal>
       state  execute 
       mine nil
     dont-turn nil

     )
     
  (p mine-gone-bad-dist-turn-right
     =goal>
       isa goal
       state correct
     =visual>
       isa ship
     > dist 125
     > angle 20
     ?imaginal>
       state free
     
         ?finger-check>
       left-index free
       left-middle free
     left-ring free
     ==>
     =visual-search> vs-dummy
     
     +imaginal-action>
     isa generic-action
     action reorbit-from-dist
     
     +manual>
     isa hold-until
     hand left
     finger index
     slots (angle)
     values (12)
     
      +goal>
       state  execute 
       mine nil
     dont-turn nil

     )
     
 (p mine-gone-bad-dist-turn-left
     =goal>
       isa goal
       state correct
     =visual>
       isa ship
     > dist 125
     < angle -20
     ?imaginal>
       state free
     
          ?finger-check>
       left-index free
       left-middle free
    left-ring free
    ==>
     =visual-search> vs-dummy
     
     +imaginal-action>
     isa generic-action
     action reorbit-from-dist
     
     +manual>
     isa hold-until
     hand left
     finger ring
     slots (angle)
     values (-12)
     
      +goal>
       state  execute 
       mine nil
     dont-turn nil

     ) 
    
    
  ;; Detecting that the mine is gone is
  ;; a high priority action
  
  (spp mine-gone-check-path :u 40)

  ;;; When the model is going back to play
  ;;; after shooting at a mine and there is
  ;;; a fortress it needs to reacquire the
  ;;; vlnr info.  It's going to do that 
  ;;; early in a sweep -- when the vdir is
  ;;; less than 90.
  
  (p check-vlner
     =goal>
       isa goal 
       state play
     
     =visual>
       isa ship 
       < vdir 90
     
     =imaginal>
       isa game-state
       shot-count unknown
     
     =visual-search>
     isa sf-visual-location
     kind fortress
     ?imaginal>
     state free
     ==>
     =imaginal>
     
     =visual-search> vs-dummy
     
     +visual>
       isa move-attention
       screen-pos vlner-loc
     +goal>
       state update-vlner)
  
  ;;; Pretty important to do...
  
  (spp check-vlner :u 50)
  
  
  (p restore-vlner
     =goal>
       isa goal
       state update-vlner
     =visual>
       isa text
       value =val
     =visual-search>
       isa sf-visual-location
       kind ship
     =imaginal>
       isa game-state
     ?imaginal>
     state free
     ==>
     =visual-search> vs-dummy
     
     ;; why was this still an = in the BOLD model??
     
     +imaginal>
       shot-count =val
     +goal>
       state play
     +visual>
       isa attend-and-track
       screen-pos =visual-search)
    
  
  ;;; These productions attempt to correct the
  ;;; position when the ship drifts too far out while
  ;;; shooting a mine.
  ;;; Just going to go with a simple long thrust toward 
  ;;; the center for now and assume the normal productions 
  ;;; can fix it from there.
   
  
  (p correction-over
     =goal>
       isa goal
       state correct-over

     ?finger-check>
     left-middle free
     
     ?imaginal>
       state free
     ==>
     
     ;; create a new game-state and prepare to
     ;; read the current vlnr
     
     +imaginal>
       isa game-state
       shot-count unknown 
       pos 0
     ;; return the goal to the standard orbit
     +goal>
       state play
       mine nil
       dont-turn nil)
  
  
  (p corrective-turn
      =goal>
        isa goal
        state execute
          
     ?manual>
       processor free
     
     
     =imaginal>
     isa reorbit
     turn =delay
     finger =finger
     ?manual>
       processor free
     ?finger-check>
       left-index free
       left-ring free
     ?imaginal>
     state free
     
     
     ==>
     
     +imaginal>
       turn nil
     +manual>
       isa delayed-punch
       hand left
     finger =finger
     delay =delay)
  
    
  (p corrective-thrust
      =goal>
        isa goal
        state execute
     
     ?manual>
       processor free
     
     =imaginal>
     isa reorbit
     turn nil
     thrust =duration

    
     ?finger-check>
     left-index free
     left-middle free
     left-ring free
     
     ?imaginal>
     state free
    
     ==>
     ;; create a new game-state and prepare to
     ;; read the current vlnr
     
     +imaginal>
       isa game-state
       shot-count unknown 
       pos 0
     ;; return the goal to the standard orbit
     +goal>
       state play
       mine nil
     dont-turn nil
     
     +manual>
     isa hold-until
     hand left
     finger middle
     slots ( vel speed-change)
     values ((> 1) 1))
  
  

)

#|
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
|#
