#Version 1.4 - simulator, conditions, etc...
#Ben Poole
#Carnegie Mellon University
#
#Version 1.3 - config file, os dependent pathing (py2app nests the CWD three levels in)
#Version 1.2 - ship doesn't reset, more aggressive fortress, faster mines, faster ship turning
#Pygame Space Fortress
#Marc Destefano
#Rensselaer Polytechnic Institute

from __future__ import division
import sf_object
import score
import frame
import pygame
import sounds
import bonus
import os
import math
import time
import datetime
import sys
import random
import copy
import socket
from timer import Timer
from timer_old import Timer_Old
from pygame.locals import *

pygame.init()

def read_globalconf(conf_file):
    app_path = '../'
    configfile = open(os.path.join(app_path, conf_file))
    configlog = configfile.readlines()
    config = {}
    for line in configlog:
        if line[0] in ["#", "\n","\r","\t"]:
            pass
        else:
            command = line.split()
            if len(command) > 2:
                config[command[0]] = command[1:]
            else:
                config[command[0]] = command[1]
    configfile.close()
    return config


class World(object):
    """Main game application"""
    def __init__(self, global_conf, conf_file):
        super(World, self).__init__()
        self.app_path = '../'
        self.datapath = os.path.join(self.app_path, "data/")
        self.config = global_conf
        configfile = open(os.path.join(self.app_path, conf_file))
        configlog = configfile.readlines()
        for line in configlog:
            if line[0] in ["#", "\n"]:
                pass
            else:
                command = line.split()
                if len(command) > 2:
                    self.config[command[0]] = command[1:]
                else:
                    self.config[command[0]] = command[1]
        configfile.close()
        self.thrust_key = eval("pygame.K_%s"%self.config["thrust_key"])
        self.left_turn_key = eval("pygame.K_%s"%self.config["left_turn_key"])
        self.right_turn_key = eval("pygame.K_%s"%self.config["right_turn_key"])
        self.fire_key = eval("pygame.K_%s"%self.config["fire_key"])
        self.IFF_key = eval("pygame.K_%s"%self.config["IFF_key"])
        self.shots_key = eval("pygame.K_%s"%self.config["shots_key"])
        self.pnts_key = eval("pygame.K_%s"%self.config["pnts_key"])
        self.SCREEN_WIDTH = 1024
        self.SCREEN_HEIGHT = 768
        self.WORLD_WIDTH = 710
        self.WORLD_HEIGHT = 626
        self.f = pygame.font.Font("fonts/freesansbold.ttf", 14)
        self.f24 = pygame.font.Font("fonts/freesansbold.ttf", 20)
        self.f96 = pygame.font.Font("fonts/freesansbold.ttf", 72)
        self.f36 = pygame.font.Font("fonts/freesansbold.ttf", 36)
        self.sounds = sounds.Sounds()
        self.bonus = bonus.Bonus()
        self.bonus.probability = float(self.config["bonus_probability"])
        self.bonus.bonus_symbol = self.config["bonus_symbol"]
        self.bonus.symbols = self.config["non_bonus_symbols"]
        #use pygame.display.list_modes() to get a sorted list of tuples. First in the list is maximum fullscreen.
        #use this to set scales appropriately
        self.screen = pygame.display.set_mode((self.SCREEN_WIDTH, self.SCREEN_HEIGHT))
        #self.screen = pygame.display.set_mode((self.SCREEN_WIDTH, self.SCREEN_HEIGHT), \
            #pygame.FULLSCREEN)
        #setting icon immediately after setting display mode
        pygame.display.set_icon(pygame.image.load("gfx/psficon.png").convert_alpha())
        pygame.mouse.set_visible(False)
        self.simulate = int(self.config["simulate"]) == 1
        self.session_number = int(self.config["session"])
        self.events = []

        self.draw = True
        self.draw_path = True
        
        self.mine_times = [1000.0*x for x in [10,30,50,70,90,110,130,150,170,-1]]
        self.mine_times.reverse()
        self.next_mine_time = self.mine_times.pop()
        self.actr = int(self.config['actr']) == 1
        self.key_release = []
        if self.simulate:
            self.game_number = 0

    
    def setup_world(self):
        """resets world for next 5-minute session"""
        self.shell_list = []
        self.missile_list = []
        self.clock = pygame.time.Clock()
        self.worldsurf = pygame.Surface((self.WORLD_WIDTH, self.WORLD_HEIGHT))
        self.worldrect = self.worldsurf.get_rect()
        self.worldrect.top = 5
        self.worldrect.centerx = self.SCREEN_WIDTH/2
        self.scoresurf = pygame.Surface((self.WORLD_WIDTH, 64))
        self.scorerect = self.scoresurf.get_rect()
        self.scorerect.top = 634
        self.scorerect.centerx = self.SCREEN_WIDTH/2
        self.gameover = False
        self.fortress = sf_object.fortress.Fortress(self)
        self.fortress.sector_size = int(self.config["fortress_sector_size"])
        self.fortress.lock_time = int(self.config["fortress_lock_time"])
        self.ship = sf_object.ship.Ship(self)
        self.ship.turn_speed = int(self.config["ship_turn_speed"])
        self.ship.max_vel = int(self.config["ship_max_vel"])
        self.ship.acceleration_factor = float(self.config["ship_acceleration"])
        self.ship.health = int(self.config["ship_hit_points"])
        self.score = score.Score(self)
        self.frame = frame.Frame(self)
        self.bighex = sf_object.hexagon.Hex(self, int(self.config["big_hex"]))
        self.smallhex = sf_object.hexagon.Hex(self,int(self.config["small_hex"]))
        self.mine = sf_object.mine.Mine(self)
        self.mine.generate_foes(int(self.config["num_foes"]))
        self.mine.speed = int(self.config["mine_speed"])
        self.mine.foe_probability = float(self.config["mine_probability"])
        self.mine.reset_time = int(self.config["mine_spawn"])
        self.mine.timeout = int(self.config["mine_timeout"]) + self.mine.reset_time
        self.vector_explosion = pygame.image.load("gfx/exp.png")
        self.vector_explosion.set_colorkey((0, 0, 0))
        self.vector_explosion_rect = self.vector_explosion.get_rect()
        self.minetimer = Timer()
        self.fortresstimer = Timer()
        self.fortressdeathtimer = Timer()
        self.intervaltimer = Timer()
        self.intervalflag = False
        self.updatetimer = Timer()
        self.bonustimer = Timer()
        self.bonus.flag = True
        self.fortress.exists = int(self.config["fortress_enable"]) == 1
        self.fortress_enable = int(self.config["fortress_enable"]) == 1
        self.fortress.alive = self.fortress_enable
        self.mine.exists = int(self.config["mine_enable"]) == 1
        self.bonus.exists = int(self.config["bonus_enable"]) == 1
        self.bonus_select = int(self.config["bonus_select"]) == 1
            
    def wait_no_carat(self):
        if self.actr:
            channel = self.config['socket']

            events = []
            try:
                keyrelease = self.key_release.pop(0)
            except IndexError:
                keyrelease = False


            waiting = True
            while waiting:
            
                try:
                    foo = channel.recv(7)
#                    print 'Got [', foo, ']'
                    
                    waiting = (foo.strip() != 'ready')
                    if waiting:
                        try:
                            key = ord(foo[0])
                            act = int(foo[1])
                        except:
                            key = 0
                            act = 0
                        if act == 1:
                            self.key_release.append(self.to_event([3,key]))
                            act = 2

                        events.append(self.to_event([act, key]))      
                except socket.error:
                    waiting = True
                  
#            print 'Next step'
        else:
            shift_flag = False
            pygame.event.clear()

            while True:
                for event in pygame.event.get():
                    if event.type == pygame.KEYDOWN:
                        if event.key == pygame.K_ESCAPE:
                            sys.exit()
                        if event.key in (pygame.K_LSHIFT,pygame.K_RSHIFT):
                            shift_flag = True
                        if not (event.key in (pygame.K_CARET,) or (shift_flag and event.key == pygame.K_6)):
                            #self.slog.write('%f got-carat-%d\n'%(time.time(),self.game_number))
                            return

    def display_experiment(self,start_time):
        """before each game, display task"""
        if self.mine.exists:
           condition = "+"
        else:
           condition = "-"
        condition += "mines, "
        if self.fortress.exists:
            condition += "+"
        else:
            condition += "-"
        condition += "fortress"
        self.screen.fill((0,0,0))
        top = self.f24.render("The task for this game is: ",True,(255,255,0))
        top_rect = top.get_rect()
        top_rect.centerx = self.SCREEN_WIDTH/2
        top_rect.centery = 270
        middle = self.f96.render(condition, True, (255,255,255))
        middle_rect = middle.get_rect()
        middle_rect.centerx = self.SCREEN_WIDTH/2
        middle_rect.centery =self.SCREEN_HEIGHT/2
        midbot = self.f24.render("", True, (255,255,0))
        midbot_rect = midbot.get_rect()
        midbot_rect.centerx = self.SCREEN_WIDTH/2
        midbot_rect.centery = 500
        #if self.image:
        #    dstr = "Press any key to begin"
        #else:
        dstr = ""
        bottom = self.f24.render(dstr, True, (255,255,0))
        bottom_rect = bottom.get_rect()
        bottom_rect.centerx = self.SCREEN_WIDTH/2
        bottom_rect.centery = 600
        self.screen.blit(top, top_rect)
        self.screen.blit(middle, middle_rect)
        self.screen.blit(midbot, midbot_rect)
        self.screen.blit(bottom, bottom_rect)
        pygame.display.flip()
        self.log.write("# %f %d Game condition (Mines Fortress): %r %r\n0 "%(time.time(), pygame.time.get_ticks(), self.mine.exists, self.fortress.exists))

        if self.actr:
            if self.mine.exists:
               (self.config['socket']).send(":mine-true\r\n");
            else:
               (self.config['socket']).send(":mine-false\r\n");

            if self.fortress.exists:
               (self.config['socket']).send(":fortress-true\r\n");
            else:
               (self.config['socket']).send(":fortress-false\r\n");

            (self.config['socket']).send(":start %d\r\n"%(start_time));
            self.wait_for_ready()
        else:
            self.wait_no_carat()
   #     self.delay_and_log(int(self.config['condition_time']));

    def display_fixation(self):
        """before each game, display task"""
        self.screen.fill((0,0,0))
        middle = self.f96.render("+", True, (255,255,255))
        middle_rect = middle.get_rect()
        middle_rect.centerx = self.SCREEN_WIDTH/2
        middle_rect.centery =self.SCREEN_HEIGHT/2
        #if self.image:
        #    dstr = "Press any key to begin"
        #else:
        self.screen.blit(middle, middle_rect)
        pygame.display.flip()
   #     self.delay_and_log(int(self.config['fixation_time']));

    def display_pause(self):
        """before each game, display task"""
        self.screen.fill((0,0,0))
        middle = self.f36.render("Session %d Block %d/%d Complete."%(self.session_number, self.game_number/2, int(self.config['games_per_session'])/2), True, (255,255,255))
        middle_rect = middle.get_rect()
        middle_rect.centerx = self.SCREEN_WIDTH/2
        middle_rect.centery =self.SCREEN_HEIGHT/2
        midbot = self.f24.render("", True, (255,255,0))
        midbot_rect = midbot.get_rect()
        midbot_rect.centerx = self.SCREEN_WIDTH/2
        midbot_rect.centery = 500
        bottom = self.f24.render("Press any key to continue", True, (255,255,0))
        bottom_rect = bottom.get_rect()
        bottom_rect.centerx = self.SCREEN_WIDTH/2
        bottom_rect.centery = 600
        self.screen.blit(middle, middle_rect)
        self.screen.blit(midbot, midbot_rect)
        self.screen.blit(bottom, bottom_rect)
        pygame.display.flip()
        self.wait_no_carat()


    def display_wait(self):
        """before each game, display task"""
        self.screen.fill((0,0,0))
        middle = self.f36.render("Waiting...", True, (255,255,255))
        middle_rect = middle.get_rect()
        middle_rect.centerx = self.SCREEN_WIDTH/2
        middle_rect.centery =self.SCREEN_HEIGHT/2
        midbot = self.f24.render("", True, (255,255,0))
        midbot_rect = midbot.get_rect()
        midbot_rect.centerx = self.SCREEN_WIDTH/2
        midbot_rect.centery = 500
        bottom = self.f24.render("Press ^ to continue", True, (255,255,0))
        bottom_rect = bottom.get_rect()
        bottom_rect.centerx = self.SCREEN_WIDTH/2
        bottom_rect.centery = 600
        self.screen.blit(middle, middle_rect)
        self.screen.blit(midbot, midbot_rect)
        self.screen.blit(bottom, bottom_rect)
        pygame.display.flip()
        shift_flag = False
        while True:
            for event in pygame.event.get():
                if event.type == pygame.KEYDOWN:
                    if event.key == pygame.K_ESCAPE:
                        sys.exit()
                    if event.key in (pygame.K_LSHIFT,pygame.K_RSHIFT):
                        shift_flag = True
                    if event.key in (pygame.K_CARET,) or (shift_flag and event.key == pygame.K_6):
                        self.slog.write('%f got-carat-%d\n'%(time.time(),self.game_number))
                        return

    def delay_and_log(self,ttime):
        start = time.time()
        end = start + ttime/1000.0
        shift_flag = False
        while time.time() < end:
            for event in pygame.event.get():
                if event.type == pygame.KEYDOWN:
                    if event.key == pygame.K_ESCAPE:
                        sys.exit()
                    if event.key in (pygame.K_LSHIFT,pygame.K_RSHIFT):
                        shift_flag = True
                    if event.key in (pygame.K_CARET,) or (shift_flag and event.key == pygame.K_6):
                        self.slog.write('%f got-carat-%d\n'%(time.time(),self.game_number))

    def display_foe_mines(self,start_time):
        """before game begins, present the list of IFF letters to target"""
        self.screen.fill((0,0,0))
        top = self.f24.render("The FOE mines for this session are:", True, (255,255,0))
        top_rect = top.get_rect()
        top_rect.centerx = self.SCREEN_WIDTH/2
        top_rect.centery = 270
        middle = self.f96.render(", ".join(self.mine.foe_letters), True, (255,255,255))
        middle_rect = middle.get_rect()
        middle_rect.centerx = self.SCREEN_WIDTH/2
        middle_rect.centery =self.SCREEN_HEIGHT/2
        midbot = self.f24.render("Try to memorize them before proceeding", True, (255,255,0))
        midbot_rect = midbot.get_rect()
        midbot_rect.centerx = self.SCREEN_WIDTH/2
        midbot_rect.centery = 500
        #if self.image:
        #    dstr = "Press any key to begin"
        #else:
        dstr = ""
        bottom = self.f24.render(dstr, True, (255,255,0))
        bottom_rect = bottom.get_rect()
        bottom_rect.centerx = self.SCREEN_WIDTH/2
        bottom_rect.centery = 600
        self.screen.blit(top, top_rect)
        self.screen.blit(middle, middle_rect)
        self.screen.blit(midbot, midbot_rect)
        self.screen.blit(bottom, bottom_rect)
        pygame.display.flip()
        self.log.write("# %f %d Foe mines: %s\n"%(time.time(), pygame.time.get_ticks(), " ".join(self.mine.foe_letters)))

        if self.actr:
            (self.config['socket']).send(":foes %s\r\n"%(" ".join(self.mine.foe_letters)));
            (self.config['socket']).send(":start %d\r\n"%(start_time));
            self.wait_for_ready()
        else:
            self.wait_no_carat()
              
        
        
   #     self.delay_and_log(int(self.config['foe_mine_time']));

    

    def to_event(self,foo):
        return pygame.event.Event(foo[0], key=foo[1]);

    def ladd(self, e):
        self.events.append(e)

    def process_input_events(self):
        """chief function to process keyboard events"""
        keys = []
        self.keys_pressed = []
        if self.simulate:
            events = map(self.to_event,self.keys)
            for event in pygame.event.get():
                #print(event)
                if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
                    sys.exit();
        elif self.actr:
            channel = self.config['socket']

            events = []
            try:
                keyrelease = self.key_release.pop(0)
            except IndexError:
                keyrelease = False


            waiting = True
            while waiting:
            
                try:
                    foo = channel.recv(7)
#                    print 'Got [', foo, ']'
                    
                    waiting = (foo.strip() != 'ready')
                    if waiting:
                        try:
                            key = ord(foo[0])
                            act = int(foo[1])
                        except:
                            key = 0
                            act = 0
                        if act == 1:
                            self.key_release.append(self.to_event([3,key]))
                            act = 2

                        events.append(self.to_event([act, key]))      
                except socket.error:
                    waiting = True
                  
#            print 'Next step'
            if keyrelease:
                events.append(keyrelease)

        else:
            events = pygame.event.get();
        for event in events:
            #print event
            if event.type == pygame.KEYDOWN or event.type == pygame.KEYUP:
                keys.append([event.type, event.key]);
            if event.type == pygame.KEYDOWN:
                self.keys_pressed.append(event.key)
                if event.key in (K_CARET,):
                    self.slog.write('%f got-carat-%d\n'%(time.time(),self.game_number))
                if event.key in (K_LMETA, K_RMETA, K_LALT, K_RALT, K_TAB):
                    print('PLEASE RETURN TO THE GAME IMMEDIATELY!!')
                if event.key == pygame.K_ESCAPE:
                    self.log.write("# Escaped prematurely\n")
                    self.evt.write("# Escaped prematurely\n")
                    self.log.close()
                    self.evt.close()
                    if not self.simulate:
                        self.keys.write("#ESCAPE\n");
                        self.keys.close()
                        for f in ('dat','key','evt'):
                            os.rename(os.path.join(self.datapath, "%s-%d-%d.%s"%(self.config["id"], self.session_number, self.game_number,f)), \
                            os.path.join(self.datapath, "short-%s-%d-%d.%s"%(self.config["id"], self.session_number, self.game_number,f)))
                    sys.exit()
                if event.key == self.right_turn_key:
                    self.ship.turn_flag = 'right'
                    self.ladd('hold-right')
                if event.key == self.left_turn_key:
                    self.ship.turn_flag = 'left'
                    self.ladd('hold-left')
                if event.key == self.thrust_key:
                    self.ship.thrust_flag = True
                    self.ladd('hold-thrust')
                if event.key == self.fire_key and (self.mine.exists or self.fortress.exists):
                    self.missile_list.append(sf_object.missile.Missile(self))
                    self.ladd('hold-fire')
                    #if self.draw:
                    #    self.sounds.missile_fired.play()
                    if self.fortress.exists:
                        if self.score.shots > 0:
                            self.score.shots -= 1
                        else:
                            self.score.pnts -= int(self.config["missile_penalty"])
                if event.key == self.IFF_key:
                    self.ladd('hold-iff')
                    if self.intervalflag == False:
                        self.intervaltimer.reset()
                        self.intervalflag = True
                    else:
                        self.score.intrvl = self.intervaltimer.elapsed()
                        self.intervalflag = False
                if event.key == self.shots_key and self.bonus.exists:
                    self.ladd('hold-shots')
                    if self.bonus.current_symbol == self.bonus.bonus_symbol and self.bonus.prior_symbol == self.bonus.bonus_symbol and self.bonus.flag:
                        self.bonus.current_symbol = "Bonus"
                        if self.bonus_select and self.score.shots > 50:
                            self.ladd('got-bonus-pnts')
                            self.score.pnts += 100
                        else:
                            self.ladd('got-bonus-shots')
                            self.score.shots += 50
                            if self.score.shots > 100:
                                self.score.shots = 100
                    elif self.bonus.current_symbol == self.bonus.bonus_symbol and self.bonus.prior_symbol != self.bonus.bonus_symbol:
                        self.bonus.flag = False
                if event.key == self.pnts_key and self.bonus.exists:
                    self.ladd('hold-pnts')
                    if self.bonus.current_symbol == self.bonus.bonus_symbol and self.bonus.prior_symbol == self.bonus.bonus_symbol and self.bonus.flag:
                        self.ladd('got-bonus-pnts')
                        self.bonus.current_symbol = "Bonus"
                        self.score.pnts += 100
                    elif self.bonus.current_symbol == self.bonus.bonus_symbol and self.bonus.prior_symbol != self.bonus.bonus_symbol:
                        self.bonus.flag = False
            if event.type == pygame.KEYUP:
                if event.key == self.left_turn_key:
                    self.ladd('release-left')
                    if self.ship.turn_flag == 'left':
                        self.ship.turn_flag = False
                if event.key == self.right_turn_key:
                    self.ladd('release-right')
                    if self.ship.turn_flag == 'right':
                        self.ship.turn_flag = False
                if event.key == self.thrust_key:
                    self.ladd('release-thrust')
                    self.ship.thrust_flag = False
                if event.key == self.fire_key:
                    self.ladd('release-fire')
                if event.key == self.IFF_key:
                    self.ladd('release-iff')
                if event.key == self.shots_key:
                    self.ladd('release-shots')
                if event.key == self.pnts_key:
                    self.ladd('release-pnts')
        if not self.simulate:

            b = [self.tinc]
            b.append(keys)
            self.keys.write(str(b))
            #self.keys.write("\t# " + str(system_clock) + "\t" + str(game_time));
            #self.keys.write("\n");
                

    def send_world(self,cur_time):
        """chief function to update the world"""
        if self.config['actr']:
            #print "x:%d y:%d ang:%d speed:%g dist:%d vdir:%d\r\n"%(self.ship.position.x, self.ship.position.y,self.ship.angle_to_fortress(), self.ship.speed(), self.ship.dist(), self.ship.vdir())
            if self.ship.alive:
                (self.config['socket']).send(":ship %d %d %d %g %d %d %d %g %g\r\n"%(self.ship.position.x, self.ship.position.y,self.ship.angle_to_fortress(), self.ship.speed(), self.ship.dist(), self.ship.vdir(), self.ship.orientation, self.ship.velocity.x, self.ship.velocity.y));
            else:
                (self.config['socket']).send(":explosion %d %d\r\n"%(self.ship.position.x, self.ship.position.y));
            if self.fortress.alive:
                (self.config['socket']).send(":fortress\r\n");
            if self.fortress.hit:
                (self.config['socket']).send(":fortress-hit\r\n");
            if self.mine.alive:
                (self.config['socket']).send(":mine %d %d\r\n"%(self.mine.position.x, self.mine.position.y));
            if not self.score.iff == '':
                (self.config['socket']).send(":iff \"%s\"\r\n"%(self.score.iff));
            if self.bonus.visible == True and not self.bonus.current_symbol == '':
                (self.config['socket']).send(":bonus \"%s\"\r\n"%(self.bonus.current_symbol));

            (self.config['socket']).send(":vlner %d\r\n"%(self.score.vlner));
            
    def send_wait(self,cur_time):
        """chief function to update the world"""
        if self.config['actr']:

            # need to send an update to go with the last world info
            (self.config['socket']).send(":start %d\r\n"%(cur_time+33));
            self.wait_for_ready()
            
            # Then can send the new "empty" update
            (self.config['socket']).send(":trial-end\r\n");
            (self.config['socket']).send(":start %d\r\n"%(cur_time+10000))
            self.wait_for_ready();
       
    def wait_for_ready(self):
        """chief function to process keyboard events"""
        channel = self.config['socket']

        waiting = True
        while waiting:
                
            try:
                foo = channel.recv(7)
                waiting = (foo.strip() != 'ready')

            except socket.error:
                waiting = True
                  

    
    def update_world(self,cur_time):
        """chief function to update the world"""
#        if self.config['actr']:
#            #print "x:%d y:%d ang:%d speed:%g dist:%d vdir:%d\r\n"%(self.ship.position.x, self.ship.position.y,self.ship.angle_to_fortress(), self.ship.speed(), self.ship.dist(), self.ship.vdir())
#            (self.config['socket']).send(":ship %d %d %d %g %d %d %d\r\n"%(self.ship.position.x, self.ship.position.y,self.ship.angle_to_fortress(), self.ship.speed(), self.ship.dist(), self.ship.vdir(), self.ship.orientation));

#        if self.intervaltimer.elapsed() > 5000:
#            self.score.intrvl = 0
#            self.intervalflag = False
        if self.updatetimer.elapsed() > int(self.config["update_timer"]):
            self.updatetimer.reset()
            if float(self.config["min_speed_threshold"]) < (self.ship.velocity.x **2 + self.ship.velocity.y **2)**0.5 < float(self.config["max_speed_threshold"]):
                self.score.vlcty += int(self.config["VLCTY_increment"])
            else:
                self.score.vlcty -= int(self.config["VLCTY_increment"])
            if self.bighex.collide(self.ship):
                self.score.cntrl += int(self.config["CNTRL_increment"])
            else:
                self.score.cntrl += int(self.config["CNTRL_increment"])/2
        if self.fortress.exists and self.fortressdeathtimer.elapsed() > 1000:
            self.fortress.alive = True
        self.ship.compute()                                  #move ship
        if self.smallhex.collide(self.ship):
            if self.ship.small_hex_flag == False: #if ship hits small hex, bounce it back and subtract 5 points
                self.ladd('hit-small-hex')
                self.ship.small_hex_flag = True
                self.ship.velocity.x = -self.ship.velocity.x
                self.ship.velocity.y = -self.ship.velocity.y
                self.score.pnts -= int(self.config["small_hex_penalty"])
        else:
            self.ship.small_hex_flag = False
        self.fortress.compute(self)                     #point fortress at ship, will fire if still for a short time
        minetimer = self.minetimer.elapsed()
        if cur_time >= self.next_mine_time  and self.mine.exists and self.mine.alive == False and not self.mine.destroyed:
            #print 'Spawing a ',
            #if self.mine.mine_list[0]:
        #       print 'foe mine @ ',
        #    else:
        #       print 'friend mine @',
        #    print time.time()

        #    print self.mine.mine_list
            if self.mine.mine_list[0]:
                mine_type = 'foe'
            else:
                mine_type = 'friend'
            self.ladd('spawn-%s'%(mine_type))
            self.next_mine_time = self.mine_times.pop()

            self.mine.reset()
        if minetimer > self.mine.timeout and self.mine.exists and not self.mine.destroyed:
            #self.log.write("# mine timed out\n")
            if self.score.iff in self.mine.foe_letters:
                #self.log.write("# foe mine timed out %g\n"%(self.minetimer.elapsed()))
                self.ladd('foe-timeout')
            else:
                self.ladd('friend-timeout')
                #self.log.write("# friend mine timed out %g\n"%(self.minetimer.elapsed()))

            self.mine.alive = False
            self.score.iff = ""
            self.score.speed -= int(self.config["mine_timeout_penalty"])
            self.minetimer.reset()
        if minetimer > self.mine.timeout and self.mine.exists and self.mine.destroyed:
            self.minetimer.reset()
            self.mine.destroyed = False
        if self.mine.exists and self.mine.alive == True:
            self.mine.compute()                                  #move mine, test to see if it hits ship
            if self.mine.test_collision(self.ship):
                if self.score.iff in self.mine.foe_letters:
                    #self.log.write("# foe mine hit ship %g\n"%(self.minetimer.elapsed()))
                    self.ladd('foe-hit-ship')
                else:
                    self.ladd('friend-hit-ship')
                    #self.log.write("# friend mine hit ship %g\n"%(self.minetimer.elapsed()))
                self.ship.take_damage()
                if not self.ship.alive:
                    self.ladd('ship-destroyed')
                self.mine.alive = False
                #self.minetimer.reset()
                self.mine.destroyed = True
                self.score.pnts -= int(self.config["mine_hit_penalty"])
                self.score.iff = ""
        elif self.mine.exists:
            self.score.intrvl = 0

        for i, shell in enumerate(self.shell_list):          #move any shells, delete if offscreen, tests for collision with ship
            shell.compute()
            if shell.position.x < 0 or shell.position.x > self.WORLD_WIDTH or shell.position.y < 0 or shell.position.y > self.WORLD_HEIGHT:
                del self.shell_list[i]
            if self.ship.alive:
                if shell.test_collision(self.ship):
                    #self.log.write("# shell hit ship\n")
                    self.ladd('shell-hit-ship')
                    del self.shell_list[i]
                    self.score.pnts -= int(self.config["shell_hit_penalty"])
                    self.ship.take_damage()
                    if not self.ship.alive:
                        self.ladd('ship-destroyed')
        for i, missile in enumerate(self.missile_list):      #move any missiles, delete if offscreen
            missile.compute()
            if missile.position.x < 0 or missile.position.x > self.WORLD_WIDTH or missile.position.y < 0 or missile.position.y > self.WORLD_HEIGHT:
                del self.missile_list[i]
            if missile.test_collision(self.mine) and self.mine.alive: #missile hits mine?
                if self.score.iff in self.mine.letters: #friendly
                    if self.intervalflag or int(self.config["intrvl_min"]) <= self.score.intrvl <= int(self.config["intrvl_max"]): #false tag
                        self.ladd('hit-falsely-tagged-friend')
                        del self.missile_list[i]
                    else:
                        #self.log.write("# hit friendly mine %g\n"%(self.minetimer.elapsed()))
                        self.ladd('hit-friend')
                        self.mine.alive = False
                        self.score.iff = ""
                        self.score.pnts += int(self.config["energize_friend"])
                        if self.fortress.exists: 
                            self.score.vlner += 1
                        #see how long mine has been alive. 0-100 points if destroyed within 10 seconds, but timer runs for 5 seconds before mine appears
                        self.score.speed += 100 - 10 * (math.floor(minetimer/1000) - 10)
                    #    self.minetimer.reset()
                        self.mine.destroyed = True
                        del self.missile_list[i]
                elif self.score.iff in self.mine.foe_letters and int(self.config["intrvl_min"]) <= self.score.intrvl <= int(self.config["intrvl_max"]): #tagged successfully? int value of "" is high
                    #self.log.write("# hit tagged foe mine %g\n"%(self.minetimer.elapsed()))
                    self.ladd('hit-tagged-foe')
                    self.mine.alive = False
                    self.score.iff = ""
                    self.score.pnts += int(self.config["destroy_foe"])
                    #see how long mine has been alive. 0-100 points if destroyed within 10 seconds
                    self.score.speed += 100 - 10 * (math.floor(minetimer/1000) - 10)
                    #self.minetimer.reset()
                    self.mine.destroyed = True
                    del self.missile_list[i]
                else: #foe mine not tagged
                    #self.log.write("# hit untagged foe mine\n")
                    self.ladd('hit-untagged-foe')
                    del self.missile_list[i]
                #self.score.intrvl = 0.0

        self.fortress.hit = False
        for i, missile in enumerate(self.missile_list):#enumerating a second time, so that when mine and fortress overlap, we only remove the missile once
            if missile.test_collision(self.fortress) and self.fortress.alive: #missile hits fortress? mine needs to be dead to hurt it
                self.fortress.hit = True
                del self.missile_list[i]
                #self.log.write("# hit fortress\n")
                self.ladd('hit-fortress')
                if self.fortresstimer.elapsed() >= int(self.config["vlner_time"]):
                    if self.fortress_enable:
                        self.score.vlner += 1
                        #self.log.write("# VLNER++\n")
                        self.ladd('vlner-increased')
                if self.fortresstimer.elapsed() < int(self.config["vlner_time"]) and self.score.vlner >= (int(self.config["vlner_threshold"]) + 1):
                    #self.log.write("# fortress destroyed\n")
                    self.ladd('fortress-destroyed')

                    self.fortress.alive = False
                    self.score.pnts += int(self.config["destroy_fortress"])
                    self.score.vlner = 0
                    #if self.draw:
                    #    self.sounds.explosion.play()
                    #self.minetimer.reset()
                    #self.mine.alive = False
                    #self.score.iff = ""
                    self.ship.alive = True
                    self.fortressdeathtimer.reset()
                    #self.reset_position()
                if self.fortresstimer.elapsed() < int(self.config["vlner_time"]) and self.score.vlner < (int(self.config["vlner_threshold"]) + 1):
                    #self.log.write("# VLNER reset\n")
                    self.ladd('vlner-reset')
                    self.score.vlner = 0
                    #if self.draw:
                    #    self.sounds.vlner_reset.play()
                self.fortresstimer.reset()
            elif missile.test_collision(self.fortress):
                del self.missile_list[i]
        if self.bonus.exists and (self.bonus.visible == False) and (self.bonustimer.elapsed() >= int(self.config["symbol_down_time"])): #original span is 25 frames, time for new symbol
            self.bonus.visible = True
            self.bonustimer.reset()
            self.bonus.prior_symbol = self.bonus.current_symbol
            self.bonus.get_new_symbol()
            #self.log.write("# new symbol %s\n"%self.bonus.current_symbol)
            if self.bonus.current_symbol == self.bonus.bonus_symbol:
                self.ladd('new-symbol-bonus')
            else:
                self.ladd('new-symbol-non-bonus')
            #self.ladd('new-symbol-%s'%self.bonus.current_symbol)
        elif self.bonus.exists and (self.bonus.visible == True) and (self.bonustimer.elapsed() >= int(self.config["symbol_up_time"])): #keep symbol visible for 75 frames
            self.bonus.visible = False
            #self.log.write("# symbol disappeared\n")
            self.ladd('symbol-disappeared')
            self.bonustimer.reset()
            
    def reset_position(self):
        """pauses the game and resets"""
        if not self.simulate:
            #if self.draw:
            #    self.sounds.explosion.play()
            #pygame.time.delay(1000)
            if not self.actr:
                self.delay_and_log(1000)
        #w.minetimer.tick(1000)
        #w.updatetimer.tick(1000)
        #w.bonustimer.tick(1000)
        #w.intervaltimer.tick(1000)
        #w.fortresstimer.tick(1000)
        #w.fortressdeathtimer.tick(1000)
        w.fortress.timer.reset()

        #self.minetimer.reset()
        #self.mine.alive = False
        self.score.iff = ""
        self.ship.alive = True
        self.fortress.alive = True
        self.ship.position.x = 245
        self.ship.position.y = 315
        self.ship.velocity.x = 0
        self.ship.velocity.y = 0
        self.ship.orientation = 90
    
    
    def draw_world(self):
        """chief function to draw the world"""
        self.screen.fill((0,0,0))
        self.worldsurf.fill((0,0,0))
        self.scoresurf.fill((0,0,0))
        self.frame.draw(self.worldsurf, self.scoresurf)
        for shell in self.shell_list:
            shell.draw(self.worldsurf)
        for missile in self.missile_list:
            missile.draw(self.worldsurf)
        #draws a small black circle under the fortress so we don't see the shell in the center
        pygame.draw.circle(self.worldsurf, (0,0,0), (355,315), 30)
        if self.fortress.exists:
            if self.fortress.alive:
                self.fortress.draw(self.worldsurf)
            else:
                self.vector_explosion_rect.center = (self.fortress.position.x, self.fortress.position.y)
                self.worldsurf.blit(self.vector_explosion, self.vector_explosion_rect)
        if self.ship.alive:
            self.ship.draw(self.worldsurf)
        else:
            self.vector_explosion_rect.center = (self.ship.position.x, self.ship.position.y)
            self.worldsurf.blit(self.vector_explosion, self.vector_explosion_rect)
        self.score.draw(self.scoresurf)
        self.bighex.draw(self.worldsurf)
        self.smallhex.draw(self.worldsurf)
        if self.mine.alive:
            self.mine.draw(self.worldsurf)
        if self.bonus.exists and self.bonus.visible:
            self.bonus.draw(self.worldsurf)
        self.screen.blit(self.worldsurf, self.worldrect)
        self.screen.blit(self.scoresurf, self.scorerect)
    
    def log_world(self):
        """log current frame's data to text file. Note that first line contains foe mine designations
        format:
        system_clock game_time ship_alive? ship_x ship_y ship_vel_x ship_vel_y ship_orientation mine_alive? mine_x mine_y 
        fortress_alive? fortress_orientation [missile_x missile_y ...] [shell_x shell_y ...] bonus_symbol
        pnts cntrl vlcty vlner iff intervl speed shots thrust_key left_key right_key fire_key iff_key shots_key pnts_key"""
        system_clock = time.time()
        game_time = pygame.time.get_ticks()
        if self.ship.alive:
            ship_alive = "y"
            ship_x = "%.3f"%(self.ship.position.x)
            ship_y = "%.3f"%(self.ship.position.y)
            ship_vel_x = "%.3f"%(self.ship.velocity.x)
            ship_vel_y = "%.3f"%(self.ship.velocity.y)
            ship_orientation = "%.3f"%(self.ship.orientation)
        else:
            ship_alive = "n"
            ship_x = "-"
            ship_y = "-"
            ship_vel_x = "-"
            ship_vel_y = "-"
            ship_orientation = "-"
        if self.mine.alive:
            mine_alive = "y"
            mine_x = "%.3f"%(self.mine.position.x)
            mine_y = "%.3f"%(self.mine.position.y)
        else:
            mine_alive = "n"
            mine_x = "-"
            mine_y = "-"
        if self.fortress.alive:
            fortress_alive = "y"
            fortress_orientation = str(self.fortress.orientation)
        else:
            fortress_alive = "n"
            fortress_orientation = "-"
        missile = '['
        for m in self.missile_list:
            missile += "%.3f %.3f "%(m.position.x, m.position.y)
        missile += ']'
        shell = '['
        for s in self.shell_list:
            shell += "%.3f %.3f "%(s.position.x, s.position.y)
        shell += ']'
        if self.bonus.current_symbol == '':
            bonus = "-"
        else:
            bonus = self.bonus.current_symbol
        keys = self.keys_pressed#pygame.key.get_pressed()
        if self.thrust_key in keys:
            thrust_key = "y"
        else:
            thrust_key = "n"
        if self.left_turn_key in keys:
            left_key = "y"
        else:
            left_key = "n"
        if self.right_turn_key in keys:
            right_key = "y"
        else:
            right_key = "n"
        if self.fire_key in keys:
            fire_key = "y"
        else:
            fire_key = "n"
        if self.IFF_key in keys:
            iff_key = "y" 
        else:
            iff_key = "n"
        if self.shots_key in keys:
            shots_key = "y"
        else:
            shots_key = "n"
        if self.pnts_key in keys:
            pnts_key = "y"
        else:
            pnts_key = "n"
        self.log.write("%d %s %s %s %s %s %s %s %s %s %s %s %s %s %s %d %d %d %d %s %d %d %d %s %s %s %s %s %s %s\n"%\
        (game_time, ship_alive, ship_x, ship_y, ship_vel_x, ship_vel_y, ship_orientation, mine_alive, mine_x, mine_y, fortress_alive, fortress_orientation,\
        missile, shell, bonus, self.score.pnts, self.score.cntrl, self.score.vlcty, self.score.vlner, self.score.iff, self.score.intrvl,\
        self.score.speed, self.score.shots, thrust_key, left_key, right_key, fire_key, iff_key, shots_key, pnts_key))
    
    def find_session(self):
        """method to determine which session file to open next"""
        if not os.path.exists(self.datapath):
            os.mkdir(self.datapath)
        subject_id = self.config["id"]

        if self.simulate:
            ext = 'sim'
            self.game_number = self.game_number + 1
            tempname = os.path.join(self.datapath, "%s-%d-%d.%s"%(subject_id, self.session_number, self.game_number, ext))
            self.log = open(tempname,'w')
            keyfile = tempname[:-3]+'key'
            print("Opening: %s"%(keyfile))
            if os.path.exists(keyfile):
                self.key_log = open(keyfile)
            else:
                print('Error: Could not find key file: ' + keyfile + '\n');
                sys.exit()
            return

        #self.session_number = 1
        self.game_number = 1

        ext = 'dat'
        while True:
            tempname = os.path.join(self.datapath, "%s-%d-%d.%s"%(subject_id, self.session_number, self.game_number, ext))
            #print tempname
            if os.path.exists(tempname):
                if self.game_number == int(self.config["games_per_session"]):
                    self.game_number = 1
                    self.session_number +=1
                else:
                    self.game_number += 1
            else: #We're at the first file that doesn't exist. Need to check if *prior* game ended prematurely
                if False:# and not (self.session_number == 1 and self.game_number == 1): #is there a prior version at all?
                    if self.game_number == 1: #need last game of previous session
                        prior_game = int(self.config["games_per_session"])
                        prior_session = self.session_number - 1
                    else:
                        prior_game = self.game_number - 1
                        prior_session = self.session_number
                    #check priors for last line
                    lastname = os.path.join(self.datapath, "%s-%d-%d.%s"%(subject_id, prior_session, prior_game, ext))
                    lastfile = open(lastname).readlines()
                    if lastfile[-1].split()[1] == "Escaped": #last game played ended prematurely!
                        os.rename(lastname, os.path.join(self.datapath, "short-%s-%d-%d.%s"%(subject_id, prior_session, prior_game, ext)))
                        os.rename(lastname, os.path.join(self.datapath, "short-%s-%d-%d.key"%(subject_id, prior_session, prior_game)))
                        tempname = lastname
                        self.game_number = prior_game
                        self.session_number = prior_session
                self.log = open(tempname, "w")
                keyfile = tempname[:-3]+"key"
                evtfile = tempname[:-3]+"evt"
                self.evt = open(evtfile,'w')
                if self.simulate:
                    self.key_log = open(keyfile);
                else:
                    self.keys = open(keyfile, 'w')
                self.log.write("# log version 1.4\n")
                self.log.write("# non-hashed line notation:\n")
                self.log.write("# game_clock system_clock game_time ship_alive? ship_x ship_y ship_vel_x ship_vel_y ship_orientation mine_alive? mine_x mine_y \
fortress_alive? fortress_orientation [missile_x missile_y ...] [shell_x shell_y ...] bonus_symbol \
pnts cntrl vlcty vlner iff intervl speed shots thrust_key left_key right_key fire_key iff_key shots_key pnts_key\n")
                return
    
    def fade(self):
        """fade screen to show score"""
        fadesurf = pygame.Surface((self.SCREEN_WIDTH, self.SCREEN_HEIGHT)).convert_alpha()
        fadesurf.fill((0,0,0,6))
        for i in range(100):
            self.screen.blit(fadesurf, pygame.Rect(0,0, self.SCREEN_WIDTH, self.SCREEN_HEIGHT))
            pygame.display.flip()
    
        
    def tick(self, mspf):
        w.minetimer.tick(mspf)
        w.updatetimer.tick(mspf)
        w.bonustimer.tick(mspf)
        w.intervaltimer.tick(mspf)
        w.fortresstimer.tick(mspf)
        w.fortressdeathtimer.tick(mspf)
        w.fortress.timer.tick(mspf)



    def show_score(self, time):
        """shows score for last game and waits to continue"""
        pygame.event.get() #clear event list? Otherwise it skips
        self.screen.fill((0, 0, 0))
        sessionsurf = self.f24.render("Session %d, Game %d/%s"%(self.session_number, self.game_number, self.config["games_per_session"]), True, (255,255,255))
        sessionrect = sessionsurf.get_rect()
        sessionrect.centerx = self.SCREEN_WIDTH / 2
        sessionrect.y = 100
        self.screen.blit(sessionsurf, sessionrect)
        pntssurf = self.f24.render("PNTS score:", True, (255, 255,0))
        pntsrect = pntssurf.get_rect()
        pntsrect.move_ip((250, 200))
        self.screen.blit(pntssurf, pntsrect)
        cntrlsurf = self.f24.render("CNTRL score:", True, (255, 255,0))
        cntrlrect = cntrlsurf.get_rect()
        cntrlrect.move_ip((250, 300))
        self.screen.blit(cntrlsurf, cntrlrect)
        vlctysurf = self.f24.render("VLCTY score:", True, (255, 255,0))
        vlctyrect = vlctysurf.get_rect()
        vlctyrect.move_ip((250, 400))
        self.screen.blit(vlctysurf, vlctyrect)
        speedsurf = self.f24.render("SPEED score:", True, (255, 255,0))
        speedrect = speedsurf.get_rect()
        speedrect.move_ip((250, 500))
        self.screen.blit(speedsurf, speedrect)
        pntsnsurf = self.f24.render("%d"%self.score.pnts, True, (255, 255,255))
        pntsnrect = pntsnsurf.get_rect()
        pntsnrect.right = 700
        pntsnrect.y = 200
        self.screen.blit(pntsnsurf, pntsnrect)
        cntrlnsurf = self.f24.render("%d"%self.score.cntrl, True, (255, 255,255))
        cntrlnrect = cntrlnsurf.get_rect()
        cntrlnrect.right = 700
        cntrlnrect.y = 300
        self.screen.blit(cntrlnsurf, cntrlnrect)
        vlctynsurf = self.f24.render("%d"%self.score.vlcty, True, (255, 255,255))
        vlctynrect = vlctynsurf.get_rect()
        vlctynrect.right = 700
        vlctynrect.y = 400
        self.screen.blit(vlctynsurf, vlctynrect)
        speednsurf = self.f24.render("%d"%self.score.speed, True, (255, 255,255))
        speednrect = speednsurf.get_rect()
        speednrect.right = 700
        speednrect.y = 500
        self.screen.blit(speednsurf, speednrect)
        #draw line
        pygame.draw.line(self.screen, (255, 255, 255), (200, 580), (800, 580))
        totalsurf = self.f24.render("Total score for this game:", True, (255, 255,0))
        totalrect = totalsurf.get_rect()
        totalrect.move_ip((200, 620))
        self.screen.blit(totalsurf, totalrect)
        totalnsurf = self.f24.render("%d"%(self.score.pnts + self.score.cntrl + self.score.vlcty + self.score.speed), True, (255, 255,255))
        totalnrect = totalnsurf.get_rect()
        totalnrect.right = 700
        totalnrect.y = 620
        self.screen.blit(totalnsurf, totalnrect)

        if self.game_number == int(self.config["games_per_session"]):
            finalsurf = self.f24.render("You're done! Press any key to exit", True, (0,255,0))
        else:
            finalsurf = self.f24.render("Press any key to continue to next game or ESC to exit", True, (255,255,255))
            finalsurf = self.f24.render("", True, (255,255,255))

        finalrect = finalsurf.get_rect()
        finalrect.centerx = self.SCREEN_WIDTH /2
        finalrect.y = 700
        self.screen.blit(finalsurf, finalrect)
        pygame.display.flip()

        total = self.score.pnts + self.score.cntrl + self.score.vlcty + self.score.speed
        self.money = total / 5625.0 * self.config['bonus_per_game'] 
        self.log.write("\n# pnts score %d\n"%self.score.pnts)
        self.log.write("# cntrl score %d\n"%self.score.cntrl)
        self.log.write("# vlcty score %d\n"%self.score.vlcty)
        self.log.write("# speed score %d\n"%self.score.speed)
        self.log.write("# total score %d\n"%total)
        self.log.write("# bonus earned $%.2f\n"%self.money)
        self.log.close()
        if not self.simulate:
            self.keys.close()
        self.evt.close()
    #    slog.write("before wait\n")
        if self.actr:
            self.send_wait(time)
   #     slog.write("after wait\n")
   #     self.wait_no_carat()
    #    slog.write("after wait_no_carat\n")
    #    self.delay_and_log(int(self.config['score_time']))
        slog.write("end of show_scores")   
    
def find_session_number(config):
        app_path = '../'
        datapath = os.path.join(app_path, "data/")

        if not os.path.exists(datapath):
            os.mkdir(datapath)
        subject_id = config["id"]

        session_number = 1
        game_number = 1

        ext = 'dat'
        while True:
            tempname = os.path.join(datapath, "%s-%d-%d.%s"%(subject_id, session_number, game_number, ext))
            #print tempname
            if os.path.exists(tempname):
                if game_number == int(config["games_per_session"]):
                    game_number = 1
                    session_number +=1
                else:
                    game_number += 1
            else: #We're at the first file that doesn't exist. Need to check if *prior* game ended prematurely
                if not (session_number == 1 and game_number == 1): #is there a prior version at all?
                    if game_number == 1: #need last game of previous session
                        prior_game = int(config["games_per_session"])
                        prior_session = session_number - 1
                    else:
                        prior_game = game_number - 1
                        prior_session = session_number
                    #check priors for last line
                    lastname = os.path.join(datapath, "%s-%d-%d.%s"%(subject_id, prior_session, prior_game, ext))
                    lastfile = open(lastname).readlines()
                    if lastfile[-1].split()[1] == "Escaped": #last game played ended prematurely!
                        os.rename(lastname, os.path.join(datapath, "short-%s-%d-%d.%s"%(subject_id, prior_session, prior_game, ext)))
                        os.rename(lastname, os.path.join(datapath, "short-%s-%d-%d.key"%(subject_id, prior_session, prior_game)))
                        tempname = lastname
                        game_number = prior_game
                        session_number = prior_session
                return session_number

if __name__ == '__main__':
    
    global_configfile = "config.txt"


    gc = read_globalconf(global_configfile)
    games = int(gc["games_per_session"])
    simulate = int(gc['simulate'])
    image = int(gc['image']) == 1
    actr = int(gc['actr']) == 1
    model_time_offset = 0
    draw_screen = int(gc['draw_screen']) == 1
    draw_path = int(gc['draw_path']) == 1

    
    if actr:
#       print('Enter session number: '),
#       session = int(sys.stdin.readline()[:-1]);
        session = int(gc['session'])

        sock = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        sock.bind(('',3000))
        sock.listen(1)
        channel, details = sock.accept()
        channel.setblocking(1)
        gc['socket'] = channel


    idx = 0
    if not simulate:
        if not gc.has_key('id'):
            print(' Enter user id: '),
            gc['id'] = sys.stdin.readline()[:-1]
            print('\n')
            print('Enter session number: '),
            session = int(sys.stdin.readline()[:-1]);
        else:
            session = int(gc['session']);
            #pass;#session = find_session_number(gc)
        gc['session'] = session
        speedup = 1
    else:
        if not gc.has_key('id'):
            print(' Enter user id: '),
            gc['id'] = sys.stdin.readline()[:-1] 
        if not gc.has_key('session'):
            print('Enter session number: '),
            session = int(sys.stdin.readline()[:-1])
        else:
            session = int(gc['session'])
        if not gc.has_key('game'):
            print('Enter starting game number: '),
            idx = int(sys.stdin.readline()[:-1])-1
        else:
            idx = int(gc['game'])-1
        if not gc.has_key('speedup'):
            print('Enter speedup value: '),
            speedup = int(sys.stdin.readline()[:-1])
        else:
            speedup = int(gc['speedup'])
    gc['session'] = session


    gc['bonus_per_game'] = float(gc['bonus_per_game'])
    idx = 0

    #random.seed(session+int(hash(gc["id"])));

    if (gc["random_seed"] == "random"):
        start_seed = session+int(hash(gc["id"]))
    else:
        start_seed = int(gc["random_seed"])

    random.seed(start_seed)

    seeds = [random.randint(0,10000) for x in range(games)]
    
    
    if (gc["game_type"] == "all"):
        config = [gc["config_both"], gc["config_orbit"], gc["config_fortress"], gc["config_mines"]]
        cfgi = map(lambda x: range(4), range(int(math.ceil(games/4.0))))
        map(random.shuffle,cfgi)
        cfgs = []
        map(cfgs.extend, cfgi)
    elif (gc["game_type"] == "both_mines"):
        config = [gc["config_both"], gc["config_mines"]]
        cfgi = map(lambda x: range(2), range(int(math.ceil(games/2.0))))
        map(random.shuffle, cfgi)
        cfgs = []
        map(cfgs.extend, cfgi)
    else:
        config = [gc["config_"+gc["game_type"]]]
        cfgs = [0 for x in xrange(games)]


    
    if not simulate and gc.has_key('resume') and bool(gc['resume']) and gc.has_key('resume_game'):
        idx = int(gc['resume_game'])-1
        #w.game_number = idx

        
    random.seed(seeds[idx])
    w = World(copy.deepcopy(gc),config[cfgs[idx]])
    w.game_number = idx
    w.find_session()

    w.draw = draw_screen
    
    screenfn = os.path.join(w.datapath, '%s-%d.dat'%(gc['id'],gc['session']))
    slog = open(screenfn,'a')
    w.slog = slog

    btwn_game_time = int(gc['foe_mine_time']) + int(gc['condition_time'])+int(gc['score_time'])+int(gc['fixation_time'])
    if not simulate:
        w.keys.write('# %d\n'%start_seed)
    w.setup_world()
    slog.write('%f setup-%d\n'%(time.time(), w.game_number))
    if image:
        slog.write('%f wait-user-%d\n'%(time.time(), w.game_number))
        w.display_pause()
        slog.write('%f wait-carat-%d\n'%(time.time(), w.game_number))
        w.display_wait()
        if w.game_number % 2 == 1:
            slog.write('%f block-start-%d\n'%(time.time(),(1+w.game_number)/2))
        slog.write('%f display-fixation-%d\n'%(time.time(), w.game_number))
        w.display_fixation()
        slog.write('%f display-foe-%d\n'%(time.time(), w.game_number))
        block_start_time = time.time()

    pygame.event.clear()

    if actr:
        channel.send(":start %d\r\n"%(model_time_offset));
        w.wait_for_ready()
    
    model_time_offset = model_time_offset + 10000
        
    w.display_foe_mines(model_time_offset)

    slog.write('%f display-exp-%d\n'%(time.time(), w.game_number))

    model_time_offset = model_time_offset + 10000
    w.display_experiment(model_time_offset)
    model_time_offset = model_time_offset + 1
    
    if simulate:
        w.log.write("# %f %d Game condition: %r %r\n0 "%(time.time(), pygame.time.get_ticks(), w.mine.exists, w.fortress.exists))
    
    if simulate or actr:
        gameTimer = Timer()
    else:
        gameTimer = Timer_Old()

    fps = 30.0
    mspf = 33 #1000.0/fps
    bonus_won = 0.0

    idx = idx+1
    prev_time = 0;
    
    destroyed = False;
    slog.write('%f begin-%d\n'%(time.time(), w.game_number))

    w.draw_path =draw_path

    
    while w.gameover == False:
        cur_time = gameTimer.elapsed();
        cur_rtime = time.time();
        times = (cur_time, cur_rtime);
        stimes = (gameTimer.elapsed(), cur_rtime, w.game_number)
        if simulate:
            while True:
                line = w.key_log.readline()
                if len(line) == 0:
                    print('Error: keys file too short.')
                    w.wait_no_carat()
                    sys.exit()
                if line[0] in ["#","\n"]:
                    continue
                break;
            keyl = eval(line)
            w.tinc = keyl[0]
            w.keys = keyl[1]
            #print(w.keys)
        else:
            w.tinc = cur_time - prev_time
            prev_time = cur_time
            
        if actr:
            channel.send(":start %d\r\n"%(cur_time + model_time_offset));
        
        w.process_input_events()
        w.update_world(cur_time)
        
        if not simulate:
            w.evt.write("%d %f "%times)
            w.evt.write(",".join(w.events))
            w.evt.write("\n")
            w.keys.write(" # %d %f\n"%times)
            w.log.write("%d %f "%times)
            w.log_world()
        w.events = []
        if draw_screen:
            w.draw_world()
            pygame.display.flip()
        if actr:
            w.send_world(cur_time + model_time_offset)
        
        if actr:
            gameTimer.tick(mspf) 
            w.tick(mspf)
        else:
            gameTimer.tick(w.tinc)
            w.tick(w.tinc)
            if simulate:
                pygame.time.wait( int (w.tinc / speedup))
            
        if destroyed:
            w.fortress.timer.reset()
            destroyed = False

        if w.ship.alive == False:
            w.reset_position()
            gameTimer.tick(1000 - mspf)

            w.score.pnts -= int(w.config["ship_death_penalty"])
            destroyed = True;
        if cur_time >= int(w.config["game_time"]): 
            slog.write('%f end-game-%d\n'%(time.time(), w.game_number))
            #w.fade()
            slog.write('%f show-score-%d\n'%(time.time(), w.game_number))

            if simulate:
                w.wait_no_carat()
            
            w.show_score(cur_time + model_time_offset)
            model_time_offset = model_time_offset + 10000;
            
            slog.write('%f done-%d\n'%(time.time(), w.game_number))
            slog.write('%f display-fixation-%d\n'%(time.time(), w.game_number))
            w.display_fixation()
            
            if w.game_number % 2 == 0:
                slog.write('%f block-end-%d\n'%(time.time(),w.game_number/2))
            bonus_won = bonus_won + w.money

            if w.game_number != int(gc["games_per_session"]):
                old_game = w.game_number
                random.seed(seeds[idx])
                w = World(copy.deepcopy(gc), config[cfgs[idx]]);
                w.draw_path =draw_path
                w.slog = slog
                w.game_number = old_game
                idx = idx + 1;
                w.find_session()
                prev_time = 0.0

                w.setup_world()
                slog.write('%f setup-%d\n'%(time.time(), w.game_number))
                #if disp:
                if image and w.game_number % 2 == 1:
                    slog.write('%f wait-user-%d\n'%(time.time(), w.game_number))
                    w.display_pause()
                    slog.write('%f wait-carat-%d\n'%(time.time(), w.game_number))
                    w.display_wait()
                    slog.write('%f block-start-%d\n'%(time.time(),(1+w.game_number)/2))
                    slog.write('%f display-fixation-%d\n'%(time.time(), w.game_number))
                    w.display_fixation()

                slog.write('%f display-foe-%d\n'%(time.time(), w.game_number))

                elap = gameTimer.elapsed()
                model_time_offset = model_time_offset + elap
                
                model_time_offset = model_time_offset + 10000
    
                w.display_foe_mines(model_time_offset)
                slog.write('%f display-exp-%d\n'%(time.time(), w.game_number))

                model_time_offset = model_time_offset + 10000
                w.display_experiment(model_time_offset)
                model_time_offset = model_time_offset + 1
                if simulate:
                    w.log.write("# %f %d Game condition: %r %r\n0 "%(time.time(), pygame.time.get_ticks(), w.mine.exists, w.fortress.exists))
 
                gameTimer.reset()
                # cur_time = 0-mspf

                if image and w.game_number % 2 == 0:
                    tickamt = elap - int(gc['game_time']) - btwn_game_time
                    gameTimer.tick(tickamt)
                

                w.minetimer.reset()
                w.updatetimer.reset()
                w.bonustimer.reset()
                slog.write('%f begin-%d\n'%(time.time(), w.game_number))
            else:
                if simulate:
                    w.wait_no_carat()
                slog.close()
                bonus_won =  max(0.0, bonus_won)
                print "You earned $%.2f bonus!"%bonus_won
                w.gameover = True
                sys.exit()

"""This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.


This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.


You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"""
