(defun reset-vision-module (vis-mod)
  
  (reset-pm-module vis-mod)
  
  ;; This stuff was moved from an after method on reset-pm-module...
  
  (clrhash (visicon vis-mod))
  (clrhash (feat-table vis-mod))
  (remove-tracking vis-mod)
  (setf (last-scale vis-mod) nil)
  (set-cfs-mth vis-mod :RM-ORIG)
  (setf (synthd-objs vis-mod) (clrhash (synthd-objs vis-mod)))
  (setf (finst-lst vis-mod) nil)
  
  (setf (scene-change vis-mod) nil)
  
  (setf (last-obj vis-mod) nil)
  
  ;;(setf (attn-trace vis-mod) nil)
  
  ;; Not needed at this point (set-clof vis-mod #(0 0))
  
  (setf (loc-failure vis-mod) nil)
  (setf (attend-failure vis-mod) nil)
    
  (chunk-type visual-object screen-pos value status color height width)
  (chunk-type abstract-object value line-pos bin-pos)
  (chunk-type (abstract-letter (:include abstract-object)))
  (chunk-type (abstract-number (:include abstract-object)))
  (chunk-type (text (:include visual-object)))
  (chunk-type (empty-space (:include visual-object)))
  
  (chunk-type (line (:include visual-object)) end1-x end1-y end2-x end2-y)
  (chunk-type (oval (:include visual-object)))
  (chunk-type (cursor (:include visual-object)))
  
  (chunk-type (phrase! (:include visual-object)) objects words colors)
  
  (chunk-type visual-location screen-x screen-y distance kind color 
              value height width size)
  
  (chunk-type set-visloc-default type screen-x screen-y distance kind color 
              value height width size)
  
  (chunk-type (char-primitive (:include visual-location)) left right)
  
  (chunk-type vision-command)
  
  (unless (chunk-type-p pm-constant)
    (chunk-type pm-constant))
  (chunk-type color)
  
  (chunk-type (move-attention (:include vision-command)) screen-pos scale)
  
  (chunk-type (start-tracking (:include vision-command)))
  
  (chunk-type (assign-finst (:include vision-command)) object location)
  
  (chunk-type (clear-scene-change (:include vision-command)))

  (chunk-type (attend-and-track (:include vision-command)) screen-pos scale)
  
  (unless (chunk-type-p clear)
    (chunk-type clear))
  
  (define-chunks 
    (lowest isa pm-constant)
    (highest isa pm-constant)
    (current isa pm-constant)
    (external isa pm-constant)
    (internal isa pm-constant)
    (find-location isa vision-command)
    (move-attention isa vision-command)
    (assign-finst isa vision-command)
    (start-tracking isa vision-command)
    (attend-and-track isa vision-command)
    
    (black isa color)
    (red isa color)
    (blue isa color)
    (green isa color)
    (white isa color)
    (magenta isa color)
    (yellow isa color)
    (cyan isa color)
    (dark-green isa color)
    (dark-red isa color)
    (dark-cyan isa color)
    (dark-blue isa color)
    (dark-magenta isa color)
    (dark-yellow isa color)
    (light-gray isa color)
    (dark-gray isa color)
        
    (text isa chunk)
    (box isa chunk)
    (line isa chunk)
    (oval isa chunk)
    
    (new isa chunk)
    (clear isa chunk))
  
  
  (setf (default-spec vis-mod)
    (define-chunk-spec isa visual-location screen-x lowest :attended new)))


(defmethod move-attention-special ((vis-mod vision-module) &key location scale)
  (if (and (eq (exec-s vis-mod) 'BUSY) (not (tracked-obj-last-feat vis-mod)))
    (model-warning "Attention shift requested at ~S while one was already in progress." (mp-time))
    (progn
      (when (tracked-obj-last-feat vis-mod) (remove-tracking vis-mod))
      
      (setf (moving-attention vis-mod) t)
      (clear-attended vis-mod)
      (setf (last-scale vis-mod) scale)
      
      (setf (attend-failure vis-mod) nil)
        
      (schedule-event-relative 
         (randomize-time (move-attn-latency vis-mod))
         'encoding-complete-special
         :destination :vision
         :module :vision
         :params (list location scale t)
         :details (concatenate 'string "Encoding-complete " (symbol-name location)
                    " " (symbol-name scale))
       :output 'medium)
      
      (setf (current-marker vis-mod) location)
      ;; not being used right now (set-clof vis-mod (dmo-to-xy location))
      (change-state vis-mod :exec 'BUSY :proc 'BUSY))))


(defmethod encoding-complete-special ((vis-mod vision-module) loc scale track &key (requested t))
  
  (setf (moving-attention vis-mod) nil)
  (change-state vis-mod :exec 'free :proc 'free)
  (setf (current-marker vis-mod) loc)
  
  (let ((return-obj (if (chunk-p-fct loc) (get-obj-at-location vis-mod loc scale) nil)))
    
    (if return-obj
        (progn
          (set-attended vis-mod (chunk-visicon-entry return-obj))
          (attend-to-object vis-mod return-obj :requested requested)
          (when track
            (schedule-event-relative 0 'start-tracking :module :vision :output 'medium :destination :vision))
          return-obj)
      
      (progn
        (clear-attended vis-mod)
        (setf (last-obj vis-mod) nil)
        (setf (attend-failure vis-mod) t)
      
        (schedule-event-relative 0 'no-visual-object-found :maintenance t :module :vision :output 'high :details "No visual-object found")
      
        nil))))


(defmethod pm-module-request ((vis-mod vision-module) buffer-name chunk-spec)
  (case buffer-name
    (visual
     (when (visual-lock vis-mod)
       (setf (visual-lock vis-mod) nil)
       (schedule-event-relative 0 'unlock-device 
                                :module :vision
                                :destination :device
                                :priority :min
                                :output nil
                                :maintenance t))
       
     (case (chunk-spec-chunk-type chunk-spec)
       (clear ;; replaces the implicit clear from -visual
        (schedule-event-relative 0 'clear :module :vision :destination :vision :output 'low))
       (clear-scene-change
        (schedule-event-relative 0 'clear-scene-change :module :vision :destination :vision :output 'low))
       (start-tracking
        (schedule-event-relative 0 'start-tracking 
                                 :destination :vision
                                 :module :vision
                                 :output 'medium)
        )
       (assign-finst
        (let ((object (if (slot-in-chunk-spec-p chunk-spec 'object) 
                         (verify-single-explicit-value 
                          (chunk-spec-slot-spec chunk-spec 'object) 
                          :vision 'assign-finst 'object)
                         nil))
              (location (if (slot-in-chunk-spec-p chunk-spec 'location)
                            (verify-single-explicit-value 
                             (chunk-spec-slot-spec chunk-spec 'location) 
                             :vision 'assign-finst 'location)
                            nil)))
          
          (if (or object location)
            (schedule-event-relative 0 'assign-finst 
                                     :params (list vis-mod :object object 
                                                   :location location)
                                     :module :vision
                                     :output 'medium)
            (print-warning "An object or location is required for an assign-finst request"))))
       
       
       (visual-object
        (print-warning "Move attention requests are now done with an isa move-attention"))
       
       
       (move-attention
        (let ((sp (if (slot-in-chunk-spec-p chunk-spec 'screen-pos) 
                     (verify-single-explicit-value 
                      (chunk-spec-slot-spec chunk-spec 'screen-pos) 
                      :vision 'visual-object 'screen-pos)
                     nil))
              (scale (if (slot-in-chunk-spec-p chunk-spec 'scale)
                        (verify-single-explicit-value 
                         (chunk-spec-slot-spec chunk-spec 'scale) 
                         :vision 'visual-object 'scale)
                        nil)))
         ; (when scale
         ;     (print-warning "Scale values are not yet handled by the new vision module - ignoring it."))
          (when sp
            (if (chunk-p-fct sp)
                (if (chunk-type-subtype-p-fct (chunk-chunk-type-fct sp) 'visual-location)
                    (schedule-event-relative 0 'move-attention 
                                     :params (list vis-mod :scale scale :location sp)
                                     :details 
                                     (concatenate 'string "Move-attention " (symbol-name sp)
                                                  " " (symbol-name scale))                                     
                                             :module :vision)
                  (print-warning "screen-pos value ~s in a move-attention request was not a visual-location chunk" sp))
              (print-warning "screen-pos value ~s in a move-attention request was not a chunk" sp)))))
       
       (attend-and-track
        (let ((sp (if (slot-in-chunk-spec-p chunk-spec 'screen-pos) 
                     (verify-single-explicit-value 
                      (chunk-spec-slot-spec chunk-spec 'screen-pos) 
                      :vision 'visual-object 'screen-pos)
                     nil))
              (scale (if (slot-in-chunk-spec-p chunk-spec 'scale)
                        (verify-single-explicit-value 
                         (chunk-spec-slot-spec chunk-spec 'scale) 
                         :vision 'visual-object 'scale)
                        nil)))
         ; (when scale
         ;     (print-warning "Scale values are not yet handled by the new vision module - ignoring it."))
          (when sp
            (if (chunk-p-fct sp)
                (if (chunk-type-subtype-p-fct (chunk-chunk-type-fct sp) 'visual-location)
                    (schedule-event-relative 0 'move-attention-special 
                                     :params (list vis-mod :scale scale :location sp)
                                     :details 
                                     (concatenate 'string "Move-attention " (symbol-name sp)
                                                  " " (symbol-name scale))                                     
                                             :module :vision)
                  (print-warning "screen-pos value ~s in a move-attention request was not a visual-location chunk" sp))
              (print-warning "screen-pos value ~s in a move-attention request was not a chunk" sp)))))
       
       (t
        (print-warning "Invalid command ~a sent to the visual buffer" 
                       (chunk-spec-chunk-type chunk-spec)))))
    
    (visual-location
     (cond ((chunk-type-subtype-p-fct (chunk-spec-chunk-type chunk-spec) 'visual-location)
            (let ((nearest (if (slot-in-chunk-spec-p chunk-spec :nearest) 
                               (verify-single-explicit-value 
                                (chunk-spec-slot-spec chunk-spec :nearest) 
                                :vision 'visual-location :nearest)
                             :none))              
                  (attended (if (slot-in-chunk-spec-p chunk-spec :attended) 
                                (multiple-value-bind (val valid)
                                    (verify-single-explicit-value 
                                     (chunk-spec-slot-spec chunk-spec :attended) 
                                     :vision 'visual-location :attended)
                                  valid)
                              :none)))           
              
              (if (or (null nearest) (null attended))
                  (print-warning "Invalid value in a request to the visual-location buffer")
                
                (schedule-event-relative 0 'find-location :module :vision 
                                         :destination :vision 
                                         :details "Find-location" 
                                         :output 'medium
                                         :params (list chunk-spec)))))
           ((chunk-type-subtype-p-fct (chunk-spec-chunk-type chunk-spec) 'set-visloc-default)
            (let ((nearest (if (slot-in-chunk-spec-p chunk-spec :nearest) 
                               (verify-single-explicit-value 
                                (chunk-spec-slot-spec chunk-spec :nearest) 
                                :vision 'set-visloc-default :nearest)
                             :none))              
                  (attended (if (slot-in-chunk-spec-p chunk-spec :attended) 
                                (multiple-value-bind (val valid)
                                    (verify-single-explicit-value 
                                     (chunk-spec-slot-spec chunk-spec :attended) 
                                     :vision 'set-visloc-default :attended)
                                  valid)
                              :none))
                  (type (if (slot-in-chunk-spec-p chunk-spec 'type) 
                                (multiple-value-bind (val valid)
                                    (verify-single-explicit-value 
                                     (chunk-spec-slot-spec chunk-spec 'type) 
                                     :vision 'set-visloc-default 'type)
                                  val)
                              :none)))           
              
              (if (or (null nearest) (null attended))
                  (print-warning "Invalid value in a request to the visual-location buffer")
                (if (and type (not (eq type :none)) (not (chunk-type-p-fct type)))
                    (print-warning "Invalid type specified in set-visloc-default request.")
                  (schedule-event-relative 0 'set-visloc-default-request :module :vision 
                                           :destination :vision 
                                           :details "Set-visloc-default" 
                                           :output 'medium
                                           :priority 9 ; just below the buffer clearing by the production
                                           :params (list chunk-spec))))))
           (t (print-warning "Invalid command ~a sent to the visual-location buffer" 
                             (chunk-spec-chunk-type chunk-spec)))))))