
;;; Potential issue with the release is that it could happen mid production
;;; firing where that production tested state free on LHS and will make a
;;; request on the RHS.  Thus, this release will cause that to jam.
;;; May want to add some sort of general pending action test to productions
;;; like vision does now to lock the device.


(defstruct hold-until-module interval event table)

(defun create-hold-until-module (model-name)
  (declare (ignore model-name))
  (make-hold-until-module :table (make-hash-table :size 12 :test #'equalp)))


(defun reset-hold-until-module (module)
  (clrhash (hold-until-module-table module))
  (setf (hold-until-module-event module) nil))

(defun handle-hold-until-params (instance param)
   (cond ((consp param)
          (case (car param)
            (:hold-until-timing 
             (setf (hold-until-module-interval instance) (cdr param)))))
         (t
          (case param
            (:hold-until-timing
             (hold-until-module-interval instance))))))


(define-module-fct :hold-until 'nil
  (list
    (define-parameter :hold-until-timing
      :valid-test #'nonneg
      :warning "a number"
      :default-value 0.05
      :documentation "Time between checks of the holding conditions.")
    )
  
   :version "0.5a1"
   :documentation "Module to handle the visual checks for held keys"
   :creation 'create-hold-until-module
  :reset 'reset-hold-until-module 
  :params 'handle-hold-until-params)


(defun test-hold-until-condition (slot target last)
  (aif (buffer-read 'visual)
       (when (find slot (chunk-type-slot-names-fct (chunk-chunk-type-fct it)))
           (let ((current (chunk-slot-value-fct it slot)))
             
             ;(format t "Testing slot ~s for value ~s with last ~s and current ~s~%" slot target last current)
             
             (cond ((or (atom target) (and (listp target) (eql (car target) '=)))
                    (let ((target-val (if (atom target) target (second target))))
                           (cond ((chunk-slot-equal current target-val)
                           t)
                          ((eq :unknown last)
                           nil)
                          ((and (numberp target-val) (numberp last) (numberp current))
                           (or (and (< last target-val) (> current target-val))
                               (and (> last target-val) (< current target-val)))))))
                   
                   ((listp target)
                    (let ((op (car target))
                          (val (second target)))
                      (case op
                        (- (not (chunk-slot-equal current val)))
                        (> (and (numberp current) (numberp val) (> current val)))
                        (>= (and (numberp current) (numberp val) (>= current val)))
                        (< (and (numberp current) (numberp val) (< current val)))
                        (<= (and (numberp current) (numberp val) (<= current val)))))))))
       (when (eq last :unknown) t)))
                                           
                   
                    
(defun check-hold-until-conditions (module)
  
  (maphash (lambda (key value)
             (if (every 'test-hold-until-condition (first value) (second value) (third value))
                 (progn
                   (schedule-event-relative 0 'test-for-release :module :motor :destination :motor :params (list :hand (first key) :finger (second key)) :output 'low)
                   (remhash key (hold-until-module-table module)))
               (setf (gethash key (hold-until-module-table module))
                 (list (first value) (second value) 
                       (mapcar (lambda (x)
                                 (aif (buffer-read 'visual)
                                      (if (find x (chunk-type-slot-names-fct (chunk-chunk-type-fct it)))
                                          (chunk-slot-value-fct it x)
                                        :unknown)
                                      :unknown))
                         (first value))) 
             )))
           (hold-until-module-table module))
    
  (when (zerop (hash-table-count (hold-until-module-table module)))
    (delete-event (hold-until-module-event module))
    (setf (hold-until-module-event module) nil)))


(defun test-for-release (motor &key hand finger)
  (if (query-buffer 'manual '((processor . free)))
    (schedule-event-relative 0 'release-key :module :motor :destination :motor :params (list :hand hand :finger finger) :output 'low :priority :max)
    (schedule-event-after-module :motor 'test-for-release :module :motor :destination :motor :params (list :hand hand :finger finger) :output 'low)))

(defun handle-hold-until-request (motor chunk-spec)
  (declare (ignore motor))
  (let* ((hand (if (slot-in-chunk-spec-p chunk-spec 'hand) 
                   (verify-single-explicit-value 
                    (chunk-spec-slot-spec chunk-spec 'hand) 
                    :motor 'hold-key 'hand)
                 nil))
         (finger (if (slot-in-chunk-spec-p chunk-spec 'finger)
                     (verify-single-explicit-value 
                      (chunk-spec-slot-spec chunk-spec 'finger)
                      :motor 'hold-key 'finger)
                   nil))
         (slots (if (slot-in-chunk-spec-p chunk-spec 'slots)
                     (verify-single-explicit-value 
                      (chunk-spec-slot-spec chunk-spec 'slots)
                      :motor 'delayed-punch 'delay)
                  nil))
         (values (if (slot-in-chunk-spec-p chunk-spec 'values)
                     (verify-single-explicit-value 
                      (chunk-spec-slot-spec chunk-spec 'values)
                      :motor 'delayed-punch 'delay)
                  nil))
         (key-module (get-module :key-press))
         (hold-module (get-module :hold-until)))
    
    (if (and hand finger
             (listp slots)
             (listp values)
             (= (length slots) (length values)))
        (if (null (gethash (list hand finger) (key-press-module-table key-module)))
          (progn
            (setf (gethash (list hand finger) (key-press-module-table key-module)) :until)
            (setf (gethash (list hand finger) (hold-until-module-table hold-module)) 
                           (list slots values (mapcar (lambda (x)
                                                        (aif (buffer-read 'visual)
                                                             (if (find x (chunk-type-slot-names-fct (chunk-chunk-type-fct it)))
                                                                 (chunk-slot-value-fct it x)
                                                               :unknown)
                                                             :unknown))
                                                slots)))
            
              (unless (hold-until-module-event hold-module)
                (setf (hold-until-module-event hold-module) 
                  (schedule-periodic-event (hold-until-module-interval hold-module) 'check-hold-until-conditions
                                           :maintenance t :module :hold-until :destination :hold-until :output 'low
                                           :initial-delay (hold-until-module-interval hold-module))))
              
            (schedule-event-relative 
             0 
             'hold-key
             :destination :motor
             :params (list :hand hand :finger finger)
             :module :motor
             :output 'low))
          (print-warning "The ~S ~S finger is already being held so a hold-until is ignored."  hand finger))
      (print-warning "Invalid hold-until request with chunk-spec ~S" chunk-spec))))

(extend-manual-requests (hold-until hand finger slots values) handle-hold-until-request)