(defmethod clear ((module motor-module))
  (when (not (check-jam module))
    (change-state module :prep 'busy)
    (queue-command :time 0.050 :where (my-name module) :command 'change-state
                   :params '(:last none :prep free))
    (setf (last-prep module) nil)
    (setf (exec-queue module) (list nil nil))
    (setf (init-stamp module) -0.1)))


(defmethod reset-pm-module ((module motor-module))
  (setf (proc-s module) (list 'free 'free))
  (setf (exec-s module) (list 'free 'free))
  (setf (prep-s module) 'free)
  (setf (mode-s module) 'free)
  (setf (last-cmd module) 'none)
  
  (setf (input-q module) nil)
  (setf (last-prep module) nil)
  (setf (exec-queue module) (list nil nil))
  (setf (init-stamp module) -0.1))

(defmethod change-state ((module motor-module) &key proc exec prep last)
  (when proc 
    (if (listp proc)
        (case (car proc)
          (left
           (setf (first (proc-s module)) (second proc)))
          (right
           (setf (second (proc-s module)) (second proc)))
          (t 
           (print-warning "Invalid proc setting for the two-handed motor module: ~S" proc)))
      (print-warning "Invalid proc setting for the two-handed motor module: ~S" proc)))
  
  (when exec 
    (if (listp exec)
        (case (car exec)
          (left
           (setf (first (exec-s module)) (second exec)))
          (right
           (setf (second (exec-s module)) (second exec)))
          (t 
           (print-warning "Invalid exec setting for the two-handed motor module: ~S" exec)))
      (print-warning "Invalid exec setting for the two-handed motor module: ~S" exec)))
  
  (when prep (setf (prep-s module) prep))
  (when last (setf (last-cmd module) last))
  (if (or (eq (proc-s module) 'busy) 
          (eq (first (exec-s module)) 'busy)
          (eq (second (exec-s module)) 'busy)
          (eq (prep-s module) 'busy))
    (setf (mode-s module) 'busy)
    (setf (mode-s module) 'free))
  
  (setf (state-change module) t))


(defmethod prepare-movement ((module motor-module) (mvmt movement-style))
  (change-state module :prep 'BUSY :proc (list (hand mvmt) 'BUSY))
  (setf (fprep-time mvmt) 
        (rand-time (compute-prep-time module mvmt)))
  (setf (last-prep module) mvmt)
  (schedule-event-relative  (fprep-time mvmt) 'preparation-complete :destination (my-name module)
                           :module (my-name module) 
                           :details (format nil "PREPARATION-COMPLETE style ~s hand ~s" (type-of mvmt) (hand mvmt)))
  
  ;; send a dummy request so that the event trace gets broken out appropriately
  (schedule-module-request (if (eq (hand mvmt) 'left) 'motor-left 'motor-right) (define-chunk-spec-fct (list 'isa (type-of mvmt))) 0 :module :motor))


(defmethod initiation-complete ((module motor-module))
  
  (when (last-prep module)
    (case (hand (last-prep module))
          (left 
           (change-state module :proc '(left FREE)))
          
          (right 
           (change-state module :proc '(right FREE)))
          
          (t
           (print-warning "Non-handed action taken with the two-handed motor module: ~s" (last-prep module))))))

(defmethod preparation-complete ((module motor-module))
  (change-state module :prep 'free)
  (when (last-prep module)
    (if (exec-immediate-p (last-prep module))
        (case (hand (last-prep module))
          (left 
           (push-last (last-prep module) (first (exec-queue module))))
          
          (right 
           (push-last (last-prep module) (second (exec-queue module))))
          
          (t
           (print-warning "Non-handed action taken with the two-handed motor module: ~s" (last-prep module))))
          
      (when (and (plusp (init-stamp module))
                 (>= (mp-time) (+ (init-stamp module) (init-time module))))
        (change-state module :proc (list (hand (last-prep module)) 'FREE)))))
  (maybe-execute-movement module))


(defmethod maybe-execute-movement ((module motor-module))
  (when (and (first (exec-queue module)) (eq (first (exec-s module)) 'FREE))
    (perform-movement module (pop (first (exec-queue module)))))
  
  (when (and (second (exec-queue module)) (eq (second (exec-s module)) 'FREE))
    (perform-movement module (pop (second (exec-queue module))))))



(defmethod perform-movement ((module motor-module) (mvmt movement-style))
  
  (schedule-event-relative  (init-time module) 'INITIATION-COMPLETE :destination (my-name module)
                           :module (my-name module) 
                           :details (format nil "INITIATION-COMPLETE style ~s hand ~s" (type-of mvmt) (hand mvmt)))
  
  
  (change-state module :proc (list (hand mvmt) 'BUSY) :exec (list (hand mvmt) 'BUSY))
  
  ;;; DAN
  ;(setf (init-stamp module) (mp-time *mp*))
  
  (setf (init-stamp module) (mp-time))
  
  (setf (exec-time mvmt) (compute-exec-time module mvmt))
  (setf (finish-time mvmt) (compute-finish-time module mvmt))
  (queue-output-events module mvmt)
  (queue-finish-event module mvmt))


(defmethod queue-finish-event ((module motor-module) (mvmt movement-style))
  
  (schedule-event-relative (finish-time mvmt) 'finish-movement-special :destination (my-name module)
                           :params (list mvmt) :module (my-name module) 
                           :details (format nil "FINISH-MOVEMENT style ~s hand ~s" (type-of mvmt) (hand mvmt)))
  
  )

(defmethod finish-movement-special ((module pm-module) (mvmt movement-style))
  (change-state module :exec (list (hand mvmt) 'free))
  (maybe-execute-movement module))


(defmethod print-module-status ((mod motor-module))
  ;(command-output "  modality free         : ~S"
  ;                (eq (mode-s mod) 'free))
  ;(command-output "  modality busy         : ~S"
  ;                (eq (mode-s mod) 'busy))
  (command-output "  preparation free      : ~S"
                  (eq (prep-s mod) 'free))
  (command-output "  preparation busy      : ~S"
                  (eq (prep-s mod) 'busy))
  (command-output "  processor free        : ~S"
                  (and (eq (first (proc-s mod)) 'free)
                       (eq (second (proc-s mod)) 'free)))
  (command-output "  processor busy        : ~S"
                  (or (eq (first (proc-s mod)) 'busy)
                      (eq (second (proc-s mod)) 'busy)))
  (command-output "  execution free        : ~S"
                  (and (eq (first (exec-s mod)) 'free)
             (eq (second (exec-s mod)) 'free)))
  (command-output "  execution busy        : ~S"
                  (or (eq (first (exec-s mod)) 'busy)
            (eq (second (exec-s mod)) 'busy)))
  (command-output "  last-command          : ~S"
                  (last-cmd mod)))

(defmethod generic-state-query ((module motor-module) buffer slot value)
  (case slot
    ((state modality)
     (case value
       (busy
        (eq (mode-s module) 'busy))
       (free
        (eq (mode-s module) 'free))
       (t (print-warning 
           "Invalid query made of the ~S buffer with slot ~S and value ~S" 
           buffer slot value))))
    (execution
     (case value
       (busy
        (or (eq (first (exec-s module)) 'busy)
            (eq (second (exec-s module)) 'busy)))
       (free
        (and (eq (first (exec-s module)) 'free)
             (eq (second (exec-s module)) 'free)))
       (t (pm-warning 
           "Invalid query made of the ~S buffer with slot ~S and value ~S" 
           buffer slot value))))
    (preparation
     (case value
       (busy
        (eq (prep-s module) 'busy))
       (free
        (eq (prep-s module) 'free))
       (t (pm-warning 
           "Invalid query made of the ~S buffer with slot ~S and value ~S" 
           buffer slot value))))
    (processor
     (case value
       (busy
        (or (eq (first (proc-s module)) 'busy)
            (eq (second (proc-s module)) 'busy))
        )
       (free
        (and (eq (first (proc-s module)) 'free)
             (eq (second (proc-s module)) 'free))
        )
       (t (pm-warning 
           "Invalid query made of the ~S buffer with slot ~S and value ~S" 
           buffer slot value))))
    (last-command 
     (eql (last-cmd module) value))))

(defstruct hand-tracker manual)


(defun dummy-hand-requests (module buffer spec))

(defun check-separate-hands (module buffer slot value)
  (let* ((manual (aif (hand-tracker-manual module)
                      it
                      (setf (hand-tracker-manual module) (get-module :motor))))
         (key-press-module (get-module :key-press))
         (hand (if (or (eq buffer 'motor-left) (eq buffer 'hand-left)) 'left 'right))
         (fingers-down 
          (or (eq :down (gethash (list hand 'index) (key-press-module-table key-press-module)))
              (eq :down (gethash (list hand 'middle) (key-press-module-table key-press-module)))
              (eq :down (gethash (list hand 'ring) (key-press-module-table key-press-module)))
              (eq :down (gethash (list hand 'pinkie) (key-press-module-table key-press-module)))
              (eq :down (gethash (list hand 'thumb) (key-press-module-table key-press-module))))))
    (case slot
      (state
       (case value
         (busy (case buffer
                 (hand-left
                  (or (eq (first (exec-s manual)) 'busy) fingers-down))
                 (hand-right
                  (or (eq (second (exec-s manual)) 'busy) fingers-down))
                 (motor-left
                  ;(or fingers-down #|(eq (prep-s manual) 'busy) (eq (proc-s manual) 'busy)|# (eq (first (exec-s manual)) 'busy))
                  (eq (first (proc-s manual)) 'busy))
                 (motor-right
                  ;(or fingers-down #|(eq (prep-s manual) 'busy) (eq (proc-s manual) 'busy)|# (eq (second (exec-s manual)) 'busy))))
                  (eq (second (proc-s manual)) 'busy))))
         (free (case buffer
                 (hand-left
                  (and (eq (first (exec-s manual)) 'free) (not fingers-down)))
                 (hand-right
                  (or (eq (second (exec-s manual)) 'free) (not fingers-down)))
                 
                 (motor-left
                  ;(and (not fingers-down) #|(eq (prep-s manual) 'free) (eq (proc-s manual) 'free)|# (eq (first (exec-s manual)) 'free))
                  (eq (first (proc-s manual)) 'free))
                 (motor-right
                  ;(and (not fingers-down) #|(eq (prep-s manual) 'free) (eq (proc-s manual) 'free)|# (eq (second (exec-s manual)) 'free))
                  (eq (second (proc-s manual)) 'free))))))
      (t nil))))

(define-module-fct 'motor-buffer-trackers '(motor-left motor-right hand-left hand-right) nil
  :request 'dummy-hand-requests
  :query 'check-separate-hands 
  :creation (lambda (x) (make-hand-tracker))
  :version "0.5a" :documentation "Reports busy-free for hands separately.")